#!/usr/bin/perl -w

# name:     deformitycorrection.cgi
# function: hexapodfixator cgi script
# location: hexapodfixator/cgi/deformitycorrection.cgi
# author:   John Storrs
# date:     2/7/07
# version:  1.0


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;
use File::Temp qw(mktemp);


# main
{
   my ($query, $message, $file, $xml, $command, $valid, $save, $frameId,
       $fileSuffix, $correctionSteps, $correctionSchedule, $ringParameters,
       $strutParameters, $frameParameters, $strutLengths, $arguments, $ringId,
       $strutId, @ringId, @strutId);

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $correctionSteps = $query->param('correctionSteps');
   if (!$frameId) {
      $frameId = '';
   }

   # validate frame identifier
   $valid = 1;
   $save = 1;
   $file = "../../config/frameparameters/$frameId.xml";
   if (!$frameId || !(-f $file)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter an existing frame identifier.
EOT

   }

   if ($valid) {
      # if correction parameter not entered ...
      if (!$correctionSteps) {
         $save = 0;

         # if correction parameter definition exists ...
         $file = 
            "../../config/deformitycorrection/$frameId.xml";
         if (-f $file) {
            # get correction parameter
            $xml = importFile($file);
            $correctionSchedule = parseXml($xml);
            $correctionSteps 
               = $correctionSchedule->attribute('correctionSteps');
         } else {
            $valid = 0;

            $message = <<EOT;
Saved correction parameter for 'frameId' not found.
EOT

         }
      }
   }

   if ($save) {
      # save correction parameter
      $file = 
         "../../config/deformitycorrection/$frameId.xml";
      open(FILE, ">$file") or die("can't open $file for write");

      $xml = <<EOT;
<correctionSchedule id="$frameId" correctionSteps="$correctionSteps"/>
EOT

      print(FILE $xml);
      close(FILE) or die("can't open $file for write");
   }
  
   if ($valid) {
      # get ring and strut identifiers for frame $frameId
      @ringId = ();
      @strutId = ();
      $file = "../../config/frameparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $frameParameters = parseXml($xml);
         $ringId[0] = $frameParameters->attribute('ring1Id');
         $ringId[1] = $frameParameters->attribute('ring2Id');
         $strutId[0] = $frameParameters->attribute('strut1Id');
         $strutId[1] = $frameParameters->attribute('strut2Id');
         $strutId[2] = $frameParameters->attribute('strut3Id');
         $strutId[3] = $frameParameters->attribute('strut4Id');
         $strutId[4] = $frameParameters->attribute('strut5Id');
         $strutId[5] = $frameParameters->attribute('strut6Id');
      }

      # assemble arguments to call of computecorrections()
      $arguments = "$frameId ";
      foreach $ringId (@ringId) {
         $file = 
            "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $arguments .= $ringParameters->attribute('innerRadius') . ' ';
            $arguments .= $ringParameters->attribute('outerRadius') . ' ';
            $arguments .= $ringParameters->attribute('thickness') . ' ';
            $arguments .= 
               $ringParameters->attribute('holeCircle1Radius') . ' ';
            $arguments .= 
               $ringParameters->attribute('holeCircle1Count') . ' ';
            $arguments .= 
               $ringParameters->attribute('holeCircle2Radius') . ' ';
            $arguments .= 
               $ringParameters->attribute('holeCircle2Count') . ' ';
         }
      }
      foreach $strutId (@strutId) {
         $file = 
            "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $arguments .= $strutParameters->attribute('minLength') . ' ';
            $arguments .= $strutParameters->attribute('maxLength') . ' ';
            $arguments .= $strutParameters->attribute('diameter') . ' ';
            $arguments .= $strutParameters->attribute('screwRadius') . ' ';
            $arguments .= $strutParameters->attribute('screwLength') . ' ';
            $arguments .= $strutParameters->attribute('nutRadius') . ' ';
            $arguments .= $strutParameters->attribute('nutLength') . ' ';
            $arguments .= $strutParameters->attribute('tubeRadius') . ' ';
            $arguments .= $strutParameters->attribute('tubeLength') . ' ';
            $arguments .= $strutParameters->attribute('jointRadius') . ' ';
            $arguments .= $strutParameters->attribute('jointOffset') . ' ';
         }
      }
      $file = "../../config/strutlengths/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $strutLengths = parseXml($xml);
         $arguments .= $strutLengths->attribute('strut1Length') . ' ';
         $arguments .= $strutLengths->attribute('strut2Length') . ' ';
         $arguments .= $strutLengths->attribute('strut3Length') . ' ';
         $arguments .= $strutLengths->attribute('strut4Length') . ' ';
         $arguments .= $strutLengths->attribute('strut5Length') . ' ';
         $arguments .= $strutLengths->attribute('strut6Length') . ' ';
         $arguments .= $strutLengths->attribute('ringSpacing') . ' ';
      }
      $arguments .= $correctionSteps . ' ';

      # get temporary file name suffix
      $fileSuffix = mktemp('-XXXXXX');
      $arguments .= $fileSuffix . ' ';

      system("../../bin/computecorrections $arguments >> log");

      $message = <<EOT;
Graphics showing the deformity correction process from start to finish are
shown below. Here is the generated <a href="corrections.xml">xml correction
schedule</a>.
EOT

   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Deformity Correction</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Deformity Correction

</div>
<div id="header-nav">
<a href="deformityparameters.html">Previous</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="algorithms.html">Algorithms</a><span class="separator"> I </span><a href="software.html">Software</a><span class="separator"> I </span><a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a></div>
</div>
<div id="page-contents">
<h1>Deformity Correction</h1>

<div class="page-map">
<dl>
</dl>
</div>

EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }

   print <<EOT;
<div class="section">

<p>The deformity is corrected by adjusting the struts incrementally to bring
the reference bone segments into the required alignment. Here the number of
correction steps is entered, and a correction schedule and visualisation are
generated.</p>

<p>If you enter a frame identifier for which you have previously defined
deformity correction parameters (currently only correction steps), and leave
the data fields empty, then the saved data will be used.</p>

<!-- embedded html -->
<form action="deformitycorrection.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="frameId" size="30" type="text" value="$frameId">Frame 
identifier</p>

</fieldset>

<p>Currently the required number of correction steps must be entered. Later
dates will be used.</p>

<fieldset>

<p><input name="correctionSteps" size="8" type="text" value="$correctionSteps">
Number of correction steps</p>

</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
<td><button type="reset">Reset</button></td>
</tr>
</table>
</center>

EOT

   if ($valid) {

      print <<EOT;
<p><center>
<img src="graphics/fixator0$fileSuffix.png"></img>
<img src="graphics/fixator1$fileSuffix.png"></img>
</center></p>
<p><center>
<img src="graphics/fixator2$fileSuffix.png"></img>
<img src="graphics/fixator3$fileSuffix.png"></img>
</center></p>
<p><center>
<img src="graphics/fixator4$fileSuffix.png"></img>
<img src="graphics/fixator5$fileSuffix.png"></img>
</center></p>

EOT

   }

   print <<EOT;
</form><!-- end embedded html -->
</div>
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Deformity Correction

</div>
<div id="footer-nav">
<a href="deformitycorrection.html">Top</a><span class="separator"> I </span><a href="deformityparameters.html">Previous</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
