#!/usr/bin/perl -w

# name:     frameparameters.cgi
# function: hexapodfixator cgi script
# location: hexapodfixator/cgi/frameparameters.cgi
# author:   John Storrs
# date:     2/7/07
# version:  1.0


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;


# main
{
   my ($query, $frameParameters, $ringParameters, $frameId, $ring1Id, $ring2Id,
       $strut1Id, $strut2Id, $strut3Id, $strut4Id, $strut5Id, $strut6Id,
       $ring1CircleId, $ring2CircleId, $strut1Ring1HoleId, $strut1Ring2HoleId, 
       $strut2Ring1HoleId, $strut2Ring2HoleId, $strut3Ring1HoleId, 
       $strut3Ring2HoleId, $strut4Ring1HoleId, $strut4Ring2HoleId,
       $strut5Ring1HoleId, $strut5Ring2HoleId, $strut6Ring1HoleId,
       $strut6Ring2HoleId, $ring1HoleCount, $ring2HoleCount, $message, $file,
       $ringFile, $strutFile, $frameFile, $xml, $command, $ringId, $strutId,
       $valid, $save, @ringIds, @strutIds);

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $ring1Id = $query->param('ring1Id');
   $ring2Id = $query->param('ring2Id');
   $strut1Id = $query->param('strut1Id');
   $strut2Id = $query->param('strut2Id');
   $strut3Id = $query->param('strut3Id');
   $strut4Id = $query->param('strut4Id');
   $strut5Id = $query->param('strut5Id');
   $strut6Id = $query->param('strut6Id');

   $ring1CircleId = $query->param('ring1CircleId');
   $ring2CircleId = $query->param('ring2CircleId');

   $strut1Ring1HoleId = $query->param('strut1Ring1HoleId');
   $strut1Ring2HoleId = $query->param('strut1Ring2HoleId');
   $strut2Ring1HoleId = $query->param('strut2Ring1HoleId');
   $strut2Ring2HoleId = $query->param('strut2Ring2HoleId');
   $strut3Ring1HoleId = $query->param('strut3Ring1HoleId');
   $strut3Ring2HoleId = $query->param('strut3Ring2HoleId');
   $strut4Ring1HoleId = $query->param('strut4Ring1HoleId');
   $strut4Ring2HoleId = $query->param('strut4Ring2HoleId');
   $strut5Ring1HoleId = $query->param('strut5Ring1HoleId');
   $strut5Ring2HoleId = $query->param('strut5Ring2HoleId');
   $strut6Ring1HoleId = $query->param('strut6Ring1HoleId');
   $strut6Ring2HoleId = $query->param('strut6Ring2HoleId');

   # validate frame identifier
   $valid = 1;
   $save = 1;
   if ((!$frameId) || ($frameId =~ m/[^\w\d-]/)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter a valid frame identifier. Permitted characters in identifiers are
letters (A-Za-z), digits (0-9), hyphen (-) and underscore (_).
EOT

   } 

   if ($valid) {
      # if no frame parameters entered ...
      if (!$ring1Id && !$ring2Id && !$strut1Id && !$strut2Id && 
          !$strut3Id && !$strut5Id && !$strut5Id && !$strut6Id) {
         $save = 0;

         # if frame definition exists ...
         $frameFile = 
            "../../config/frameparameters/$frameId.xml";
         if (-f $frameFile) {
            # get saved frame parameters
            $xml = importFile($frameFile);
            $frameParameters = parseXml($xml);
            $ring1Id = $frameParameters->attribute('ring1Id');
            $ring2Id = $frameParameters->attribute('ring2Id');
            $strut1Id = $frameParameters->attribute('strut1Id');
            $strut2Id = $frameParameters->attribute('strut2Id');
            $strut3Id = $frameParameters->attribute('strut3Id');
            $strut4Id = $frameParameters->attribute('strut4Id');
            $strut5Id = $frameParameters->attribute('strut5Id');
            $strut6Id = $frameParameters->attribute('strut6Id');

            $ring1CircleId = $frameParameters->attribute('ring1CircleId');
            $ring2CircleId = $frameParameters->attribute('ring2CircleId');

            $strut1Ring1HoleId = 
               $frameParameters->attribute('strut1Ring1HoleId');
            $strut1Ring2HoleId = 
               $frameParameters->attribute('strut1Ring2HoleId');
            $strut2Ring1HoleId = 
               $frameParameters->attribute('strut2Ring1HoleId');
            $strut2Ring2HoleId = 
               $frameParameters->attribute('strut2Ring2HoleId');
            $strut3Ring1HoleId = 
               $frameParameters->attribute('strut3Ring1HoleId');
            $strut3Ring2HoleId = 
               $frameParameters->attribute('strut3Ring2HoleId');
            $strut4Ring1HoleId = 
               $frameParameters->attribute('strut4Ring1HoleId');
            $strut4Ring2HoleId = 
               $frameParameters->attribute('strut4Ring2HoleId');
            $strut5Ring1HoleId = 
               $frameParameters->attribute('strut5Ring1HoleId');
            $strut5Ring2HoleId = 
               $frameParameters->attribute('strut5Ring2HoleId');
            $strut6Ring1HoleId = 
               $frameParameters->attribute('strut6Ring1HoleId');
            $strut6Ring2HoleId = 
               $frameParameters->attribute('strut6Ring2HoleId');

            $message = <<EOT;
Saved parameters for frame '$frameId' are displayed below. You can edit and
save them again, or use the data as a template for another definition.
EOT

         } else {
            # no frame definition exists
            $valid = 0;

            $message = <<EOT;
Frame '$frameId' not found.
EOT

         }
      }

      # duplicate first ring and strut identifiers if others are empty
      if ($ring1Id && !$ring2Id) {
         $ring2Id = $ring1Id;
      }
      if ($strut1Id && !$strut2Id && !$strut3Id && 
          !$strut4Id && !$strut5Id && !$strut6Id) {
         $strut2Id = $strut1Id;
         $strut3Id = $strut1Id;
         $strut4Id = $strut1Id;
         $strut5Id = $strut1Id;
         $strut6Id = $strut1Id;
      }

      if ($valid) {
         # check no empty frame parameters
         if (!$ring1Id || !$ring2Id || !$strut1Id || !$strut2Id || 
             !$strut3Id || !$strut5Id || !$strut5Id || !$strut6Id) {
            $valid = 0;
            $save = 0;

            $message = <<EOT;
Please enter missing parameters.
EOT

         }
      }
   }

   if ($valid) {
      # check that ring and strut identifiers exist
      push(@ringIds, $ring1Id);
      if (!grep(/^$ring2Id$/, @ringIds)) {
         push(@ringIds, $ring2Id);
      }
      push(@strutIds, $strut1Id);
      if (!grep(/^$strut2Id$/, @strutIds)) {
         push(@strutIds, $strut2Id);
      }
      if (!grep(/^$strut3Id$/, @strutIds)) {
         push(@strutIds, $strut3Id);
      }
      if (!grep(/^$strut4Id$/, @strutIds)) {
         push(@strutIds, $strut4Id);
      }
      if (!grep(/^$strut5Id$/, @strutIds)) {
         push(@strutIds, $strut5Id);
      }
      if (!grep(/^$strut6Id$/, @strutIds)) {
         push(@strutIds, $strut6Id);
      }
      foreach $ringId (@ringIds) {
         $ringFile =
            "../../config/ringparameters/$ringId.xml";
         if (!(-f $ringFile)) {
            $valid = 0;
            $save = 0;
            
            $message .= <<EOT;
Ring $ringId not found.
EOT
   
         }
      }
      foreach $strutId (@strutIds) {
         $strutFile =
          "../../config/strutparameters/$strutId.xml";
         if (!(-f $strutFile)) {
            $valid = 0;
            $save = 0;
   
            $message .= <<EOT;
Strut $strutId not found.
EOT
   
         }
      }
   }

   if ($valid) {
      # conditionally set default hole circle parameters
      if (!$ring1CircleId) {
         $ring1CircleId = 2;
      }
      if (!$ring2CircleId) {
         $ring2CircleId = 2;
      }
      if (($strut1Ring1HoleId == 0) || ($strut1Ring2HoleId == 0) ||
          ($strut2Ring1HoleId == 0) || ($strut2Ring2HoleId == 0) ||
          ($strut3Ring1HoleId == 0) || ($strut3Ring2HoleId == 0) ||
          ($strut4Ring1HoleId == 0) || ($strut4Ring2HoleId == 0) ||
          ($strut5Ring1HoleId == 0) || ($strut5Ring2HoleId == 0) ||
          ($strut6Ring1HoleId == 0) || ($strut6Ring2HoleId == 0)) {
         # get saved ring hole circle parameters
         $ringFile =
            "../../config/ringparameters/$ring1Id.xml";
         $xml = importFile($ringFile);
         $ringParameters = parseXml($xml);
         if ($ring1CircleId == 1) {
            $ring1HoleCount = $ringParameters->attribute("holeCircle1Count");
         } else {
            $ring1HoleCount = $ringParameters->attribute("holeCircle2Count");
         }
         $ringFile =
            "../../config/ringparameters/$ring2Id.xml";
         $xml = importFile($ringFile);
         $ringParameters = parseXml($xml);
         if ($ring2CircleId == 1) {
            $ring2HoleCount = $ringParameters->attribute("holeCircle1Count");
         } else {
            $ring2HoleCount = $ringParameters->attribute("holeCircle2Count");
         }

         # hole ids here are 1-based
         $strut1Ring1HoleId = 2;
         $strut2Ring1HoleId = $ring1HoleCount/3;
         $strut3Ring1HoleId = $ring1HoleCount/3 + 2;
         $strut4Ring1HoleId = 2*$ring1HoleCount/3;
         $strut5Ring1HoleId = 2*$ring1HoleCount/3 + 2;
         $strut6Ring1HoleId = $ring1HoleCount;

         $strut1Ring2HoleId = $ring2HoleCount/6;
         $strut2Ring2HoleId = $ring2HoleCount/6 + 2;
         $strut3Ring2HoleId = $ring2HoleCount/2;
         $strut4Ring2HoleId = $ring2HoleCount/2 + 2;
         $strut5Ring2HoleId = 5*$ring2HoleCount/6;
         $strut6Ring2HoleId = 5*$ring2HoleCount/6 + 2;
      }
   }

   if ($save) {
      # save frame parameters
      $file = "../../config/frameparameters/$frameId.xml";
      open(FILE, ">$file") or die("can't open $file for write");

      $xml = <<EOT;
<frameParameters id="$frameId" ring1Id="$ring1Id" ring2Id="$ring2Id"
  strut1Id="$strut1Id" strut2Id="$strut2Id" strut3Id="$strut3Id"
  strut4Id="$strut4Id" strut5Id="$strut5Id" strut6Id="$strut6Id"
  ring1CircleId="$ring1CircleId" ring2CircleId="$ring2CircleId" 
  strut1Ring1HoleId="$strut1Ring1HoleId" strut1Ring2HoleId="$strut1Ring2HoleId"
  strut2Ring1HoleId="$strut2Ring1HoleId" strut2Ring2HoleId="$strut2Ring2HoleId"
  strut3Ring1HoleId="$strut3Ring1HoleId" strut3Ring2HoleId="$strut3Ring2HoleId"
  strut4Ring1HoleId="$strut4Ring1HoleId" strut4Ring2HoleId="$strut4Ring2HoleId"
  strut5Ring1HoleId="$strut5Ring1HoleId" strut5Ring2HoleId="$strut5Ring2HoleId"
  strut6Ring1HoleId="$strut6Ring1HoleId" strut6Ring2HoleId="$strut6Ring2HoleId"
/>
EOT

      print(FILE $xml);
      close(FILE) or die("can't open $file for write");

      $message = <<EOT;
Parameters for frame '$frameId' have been saved. 
Please print this page as a record.
EOT

   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Frame Parameters</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Frame Parameters

</div>
<div id="header-nav">
<a href="strutparameters.html">Previous</a><span class="separator"> I </span><a href="strutlengths.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="algorithms.html">Algorithms</a><span class="separator"> I </span><a href="software.html">Software</a><span class="separator"> I </span><a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a></div>
</div>
<div id="page-contents">
<h1>Frame Parameters</h1>

<div class="page-map">
<dl>
</dl>
</div>

EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }

   print <<EOT;
<div class="section">

<p>A fixator frame consists of 2 rings and 6 struts, connected in a hexapod
(triangulated) configuration. If required, different ring and strut types can
be mixed in the same frame.</p>

<p>Define frame parameters here, with a frame identifier for use in
following forms. The identifier could for example include the patient's name.
Permitted characters are letters (A-Za-z), digits (0-9), hyphen (-) and
underscore (_).</p>

<p>If you enter an existing frame identifier, and do not enter any frame
parameters, then parameters for the selected frame will be displayed for
inspection or editing.</p>

<form action="frameparameters.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="frameId" size="30" type="text" value="$frameId">Frame identifier</p>

</fieldset>

<p>If you are defining a new frame, and both rings are the same, you need only
enter an identifier for the first ring, leaving the second blank.</p>

<fieldset>

<p><input name="ring1Id" size="30" type="text" value="$ring1Id">Ring 1 identifier</p>

<p><input name="ring2Id" size="30" type="text" value="$ring2Id">Ring 2 identifier</p>

</fieldset>

<p>If you are defining a new frame, and all struts are the same, you need only
enter an identifier for the first strut, leaving the others blank.</p>

<fieldset>

<p><input name="strut1Id" size="30" type="text" value="$strut1Id">Strut 1 identifier</p>

<p><input name="strut2Id" size="30" type="text" value="$strut2Id">Strut 2 identifier</p>

<p><input name="strut3Id" size="30" type="text" value="$strut3Id">Strut 3 identifier</p>

<p><input name="strut4Id" size="30" type="text" value="$strut4Id">Strut 4 identifier</p>

<p><input name="strut5Id" size="30" type="text" value="$strut5Id">Strut 5 identifier</p>

<p><input name="strut6Id" size="30" type="text" value="$strut6Id">Strut 6 identifier</p>

</fieldset>

<p>For each ring, select which of the two defined hole circles will be used for
strut connection.</p>

<fieldset>

<div class="inputlist">
<p>Ring 1:
EOT

   if ($ring1CircleId == 1) {

      print <<EOT;
<input type="radio" name="ring1CircleId" checked="checked" value="1">
First circle
<input type="radio" name="ring1CircleId" value="2">
Second circle
EOT

   } else {

      print <<EOT;
<input type="radio" name="ring1CircleId" value="1">
First circle
<input type="radio" name="ring1CircleId" checked="checked" value="2">
Second circle
EOT

   }

   print <<EOT;
</p>
<p>Ring 2:
EOT

   if ($ring2CircleId == 1) {

      print <<EOT;
<input type="radio" name="ring2CircleId" checked="checked" value="1">
First circle
<input type="radio" name="ring2CircleId" value="2">
Second circle
EOT

   } else {

      print <<EOT;
<input type="radio" name="ring2CircleId" value="1">
First circle
<input type="radio" name="ring2CircleId" checked="checked" value="2">
Second circle
EOT

   }

   print <<EOT;
</p>

</fieldset>

<p>For each strut, identify the holes in each ring at which connections will be
made. Valid identifiers are numbers ranging from 1 to the defined number of
holes in the circle. The connection pattern doesn't have to be uniform, but it
must result in a properly triangulated frame. If no values are entered,
suitable defaults will be used; these can be adjusted later if required.</p>

<fieldset>

<div class="inputlist">
<p>strut 1:
<input name="strut1Ring1HoleId" size="2" type="text" value="$strut1Ring1HoleId">
Ring 1 hole identifier
<input name="strut1Ring2HoleId" size="2" type="text" value="$strut1Ring2HoleId">
Ring 2 hole identifier
</p>

<p>strut 2:
<input name="strut2Ring1HoleId" size="2" type="text" value="$strut2Ring1HoleId">
Ring 1 hole identifier
<input name="strut2Ring2HoleId" size="2" type="text" value="$strut2Ring2HoleId">
Ring 2 hole identifier
</p>

<p>strut 3:
<input name="strut3Ring1HoleId" size="2" type="text" value="$strut3Ring1HoleId">
Ring 1 hole identifier
<input name="strut3Ring2HoleId" size="2" type="text" value="$strut3Ring2HoleId">
Ring 2 hole identifier
</p>

<p>strut 4:
<input name="strut4Ring1HoleId" size="2" type="text" value="$strut4Ring1HoleId">
Ring 1 hole identifier
<input name="strut4Ring2HoleId" size="2" type="text" value="$strut4Ring2HoleId">
Ring 2 hole identifier
</p>

<p>strut 5:
<input name="strut5Ring1HoleId" size="2" type="text" value="$strut5Ring1HoleId">
Ring 1 hole identifier
<input name="strut5Ring2HoleId" size="2" type="text" value="$strut5Ring2HoleId">
Ring 2 hole identifier
</p>

<p>strut 6:
<input name="strut6Ring1HoleId" size="2" type="text" value="$strut6Ring1HoleId">
Ring 1 hole identifier
<input name="strut6Ring2HoleId" size="2" type="text" value="$strut6Ring2HoleId">
Ring 2 hole identifier
</p>

</div>

</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
</tr>
</table>
</center>

</form>
</div>
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Frame Parameters

</div>
<div id="footer-nav">
<a href="frameparameters.html">Top</a><span class="separator"> I </span><a href="strutparameters.html">Previous</a><span class="separator"> I </span><a href="strutlengths.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
