#!/usr/bin/perl -w

# name:     ringparameters.cgi
# function: hexapodfixator cgi script
# location: hexapodfixator/cgi/ringparameters.cgi
# author:   John Storrs
# date:     2/7/07
# version:  1.0


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;


# main
{
   my ($query, $ringParameters, $ringId, $innerRadius, $outerRadius,
       $thickness, $holeCircle1Radius, $holeCircle1Count, $holeCircle2Radius,
       $holeCircle2Count, $message, $file, $xml, $command, $valid, $save);

   # create CGI object
   $query = new CGI;

   # get form data
   $ringId = $query->param('ringId');
   $innerRadius = $query->param('innerRadius');
   $outerRadius = $query->param('outerRadius');
   $thickness = $query->param('thickness');
   $holeCircle1Radius = $query->param('holeCircle1Radius');
   $holeCircle1Count = $query->param('holeCircle1Count');
   $holeCircle2Radius = $query->param('holeCircle2Radius');
   $holeCircle2Count = $query->param('holeCircle2Count');

   # validate ring identifier
   $valid = 1;
   $save = 1;
   if (!$ringId || ($ringId =~ m/[^\w\d-]/)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter a valid ring identifier. Permitted characters in identifiers are 
letters (A-Za-z), digits (0-9), hyphen (-) and underscore (_).
EOT

   } 

   if ($valid) {
      # if no ring parameters entered ...
      if (!$innerRadius && !$outerRadius && !$thickness && 
          !$holeCircle1Radius && !$holeCircle1Count &&
          !$holeCircle2Radius && !$holeCircle2Count) {
         $save = 0;

         # if ring definition exists ...
         $file = "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            # get saved ring parameters
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $innerRadius = $ringParameters->attribute('innerRadius');
            $outerRadius = $ringParameters->attribute('outerRadius');
            $thickness = $ringParameters->attribute('thickness');
            $holeCircle1Radius = 
               $ringParameters->attribute('holeCircle1Radius');
            $holeCircle1Count = $ringParameters->attribute('holeCircle1Count');
            $holeCircle2Radius = 
               $ringParameters->attribute('holeCircle2Radius');
            $holeCircle2Count = $ringParameters->attribute('holeCircle2Count');

            $message = <<EOT;
Saved parameters for '$ringId' are displayed below. You can edit and save them
again, or use the data for another ring definition.
EOT

         } else {
            # no ring definition exists
            $valid = 0;

            $message = <<EOT;
Ring '$ringId' not found.
EOT

         }
      }
   }

   if ($save) {
      # save ring parameters
      $file = "../../config/ringparameters/$ringId.xml";
      open(FILE, ">$file") or die("can't open $file for write");

      $xml = <<EOT;
<ringParameters id="$ringId" innerRadius="$innerRadius"
  outerRadius="$outerRadius" thickness="$thickness"
  holeCircle1Radius="$holeCircle1Radius" holeCircle1Count="$holeCircle1Count"
  holeCircle2Radius="$holeCircle2Radius" holeCircle2Count="$holeCircle2Count"/>
EOT

      print(FILE $xml);
      close(FILE) or die("can't open $file for write");

      $message = <<EOT;
Parameters for ring '$ringId' have been saved. 
Please print this page as a record.
EOT
   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Ring Parameters</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Ring Parameters

</div>
<div id="header-nav">
<a href="povrayrenderer.html">Previous</a><span class="separator"> I </span><a href="strutparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="algorithms.html">Algorithms</a><span class="separator"> I </span><a href="software.html">Software</a><span class="separator"> I </span><a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a></div>
</div>
<div id="page-contents">
<h1>Ring Parameters</h1>

<div class="page-map">
<dl>
</dl>
</div>

EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }

   print <<EOT;
<div class="section">

<p>Fixator rings are circular rings of metal or carbon-reinforced plastic,
typically with two circular rows of uniformly spaced holes where struts and
other fixings may be attached. The number of holes is usually divisible by 6,
and the holes in the 2 rows usually coincide radially every 60 degrees.</p>

<p>Define ring parameters here, with a ring identifier for use when defining
fixator frame parameters. The identifier can be a manufacturer's part number or
other string. Permitted characters are letters (A-Za-z), digits (0-9), hyphen
(-) and underscore (_).</p>

<p>If you enter an existing ring identifier, and do not enter any ring
parameters, then the parameters for the selected ring will be displayed for
inspection or editing.</p>

<form action="ringparameters.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="ringId" size="30" type="text" value="$ringId">Ring identifier</p>

</fieldset>

<p>The inner and outer radius, and thickness, are used only when generating
fixator graphics. They do not affect fixator kinematics.</p>

<fieldset>

<p><input name="innerRadius" size="6" type="text" value="$innerRadius">Inner radius of 
circular ring</p>

<p><input name="outerRadius" size="6" type="text" value="$outerRadius">Outer radius of
circular ring</p>

<p><input name="thickness" size="6" type="text" value="$thickness">Thickness of circular
ring</p>

<p><input name="holeCircle1Radius" size="6" type="text" value="$holeCircle1Radius">Radius of
first circle of holes</p>

<p><input name="holeCircle1Count" size="6" type="text" value="$holeCircle1Count">Number of
holes in first circle</p>

<p><input name="holeCircle2Radius" size="6" type="text" value="$holeCircle2Radius">Radius of
second circle of holes</p>

<p><input name="holeCircle2Count" size="6" type="text" value="$holeCircle2Count">Number of
holes in second circle</p>

</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
</tr>
</table>
</center>

</form>
</div>
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Ring Parameters

</div>
<div id="footer-nav">
<a href="ringparameters.html">Top</a><span class="separator"> I </span><a href="povrayrenderer.html">Previous</a><span class="separator"> I </span><a href="strutparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>

EOT

}
