#!/usr/bin/perl -w

# name:     strutlengths.cgi
# function: hexapodfixator cgi script
# location: hexapodfixator/cgi/strutlengths.cgi
# author:   John Storrs
# date:     2/7/07
# version:  1.0


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;
use File::Temp qw(mktemp);


# main
{
   my ($query, $valid, $save, $frameId, $ringSpacing, $file, $xml, $arguments,
       $message, $ringId, $fileSuffix, $strutId, $frameParameters, 
       $ringParameters, $strutParameters, $strutLength, $minLength, $maxLength,
       $status, $i, $j, @ringId, @strutId, @strutLength, @circleId, 
       @ringHoleId);

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $strutLength[0] = $query->param('strut1Length');
   $strutLength[1] = $query->param('strut2Length');
   $strutLength[2] = $query->param('strut3Length');
   $strutLength[3] = $query->param('strut4Length');
   $strutLength[4] = $query->param('strut5Length');
   $strutLength[5] = $query->param('strut6Length');
   $ringSpacing = $query->param('ringSpacing');


   # validate frame identifier
   $valid = 1;
   $save = 1;
   $file = "../../config/frameparameters/$frameId.xml";
   if (!$frameId || !(-f $file)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter an existing frame identifier.
EOT

   } 

   if ($valid) {
      # if no strut length parameters entered ...
      if (!$strutLength[0] && !$strutLength[1] && !$strutLength[2] &&
          !$strutLength[3] && !$strutLength[4] && !$strutLength[5] &&
          !$ringSpacing) {
         $save = 0;

         # if strut length parameter definition exists ...
         $file = "../../config/strutlengths/$frameId.xml";
         if (-f $file) {
            # get strut length parameters
            $xml = importFile($file);
            $frameParameters = parseXml($xml);
            $strutLength[0] = $frameParameters->attribute('strut1Length');
            $strutLength[1] = $frameParameters->attribute('strut2Length');
            $strutLength[2] = $frameParameters->attribute('strut3Length');
            $strutLength[3] = $frameParameters->attribute('strut4Length');
            $strutLength[4] = $frameParameters->attribute('strut5Length');
            $strutLength[5] = $frameParameters->attribute('strut6Length');
            $ringSpacing = $frameParameters->attribute('ringSpacing');
         } else {
            $valid = 0;

            $message = <<EOT;
Saved strut length parameters for 'frameId' not found.
EOT

         }
      }
   }

   if ($valid) {
      # get ring and strut identifiers for frame $frameId
      @ringId = ();
      @strutId = ();
      $file = "../../config/frameparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $frameParameters = parseXml($xml);
         $ringId[0] = $frameParameters->attribute('ring1Id');
         $ringId[1] = $frameParameters->attribute('ring2Id');
         $strutId[0] = $frameParameters->attribute('strut1Id');
         $strutId[1] = $frameParameters->attribute('strut2Id');
         $strutId[2] = $frameParameters->attribute('strut3Id');
         $strutId[3] = $frameParameters->attribute('strut4Id');
         $strutId[4] = $frameParameters->attribute('strut5Id');
         $strutId[5] = $frameParameters->attribute('strut6Id');

         $circleId[0] = $frameParameters->attribute('ring1CircleId');
         $circleId[1] = $frameParameters->attribute('ring2CircleId');

         $ringHoleId[0][0] = $frameParameters->attribute('strut1Ring1HoleId');
         $ringHoleId[1][0] = $frameParameters->attribute('strut1Ring2HoleId');
         $ringHoleId[0][1] = $frameParameters->attribute('strut2Ring1HoleId');
         $ringHoleId[1][1] = $frameParameters->attribute('strut2Ring2HoleId');
         $ringHoleId[0][2] = $frameParameters->attribute('strut3Ring1HoleId');
         $ringHoleId[1][2] = $frameParameters->attribute('strut3Ring2HoleId');
         $ringHoleId[0][3] = $frameParameters->attribute('strut4Ring1HoleId');
         $ringHoleId[1][3] = $frameParameters->attribute('strut4Ring2HoleId');
         $ringHoleId[0][4] = $frameParameters->attribute('strut5Ring1HoleId');
         $ringHoleId[1][4] = $frameParameters->attribute('strut5Ring2HoleId');
         $ringHoleId[0][5] = $frameParameters->attribute('strut6Ring1HoleId');
         $ringHoleId[1][5] = $frameParameters->attribute('strut6Ring2HoleId');
      }

      # assemble arguments for call of setstrutlengths()
      $arguments = '';
      foreach $ringId (@ringId) {
         $file = "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $arguments .= $ringParameters->attribute('innerRadius') . ' ';
            $arguments .= $ringParameters->attribute('outerRadius') . ' ';
            $arguments .= $ringParameters->attribute('thickness') . ' ';
            $arguments .= $ringParameters->attribute('holeCircle1Radius') . ' ';
            $arguments .= $ringParameters->attribute('holeCircle1Count') . ' ';
            $arguments .= $ringParameters->attribute('holeCircle2Radius') . ' ';
            $arguments .= $ringParameters->attribute('holeCircle2Count') . ' ';
         }
      }
      $i = 0;
      foreach $strutId (@strutId) {
         $file = 
            "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $minLength = $strutParameters->attribute('minLength');
            $maxLength = $strutParameters->attribute('maxLength');
            $arguments .= $minLength . ' ';
            $arguments .= $maxLength . ' ';
            $arguments .= $strutParameters->attribute('diameter') . ' ';
            $arguments .= $strutParameters->attribute('screwRadius') . ' ';
            $arguments .= $strutParameters->attribute('screwLength') . ' ';
            $arguments .= $strutParameters->attribute('nutRadius') . ' ';
            $arguments .= $strutParameters->attribute('nutLength') . ' ';
            $arguments .= $strutParameters->attribute('tubeRadius') . ' ';
            $arguments .= $strutParameters->attribute('tubeLength') . ' ';
            $arguments .= $strutParameters->attribute('jointRadius') . ' ';
            $arguments .= $strutParameters->attribute('jointOffset') . ' ';

            # check strut lengths against limits
            if ($strutLength[$i] < $minLength) {
               $valid = 0;
               $save = 0;
               $j = $i + 1;

               $message = <<EOT;
Strut $j length ($strutLength[$i]) is less than the minimum length of this
strut ($minLength). Please correct this.
EOT

            } elsif ($strutLength[$i] > $maxLength) {
               $valid = 0;
               $save = 0;
               $j = $i + 1;

               $message = <<EOT;
Strut $j length ($strutLength[$i]) is greater than the maximum length of this
strut ($maxLength). Please correct this.
EOT

            }
            $i++;
         }
      }

      if ($valid) {
         foreach $strutLength (@strutLength) {
            $arguments .= $strutLength . ' ';
         }
         $arguments .= $ringSpacing . ' ';

         # get temporary file name suffix
         $fileSuffix = mktemp('-XXXXXX');
         $arguments .= $fileSuffix . ' ';

         # output hole circle parameters
         for $i (0..1) {
            $arguments .= $circleId[$i] . ' ';
         }
         for $i (0..5) {
            $arguments .= $ringHoleId[0][$i] . ' ';
            $arguments .= $ringHoleId[1][$i] . ' ';
         }

         # generate graphics
         $status = system("../../bin/setstrutlengths $arguments >> log");  
         if (($status >> 8) == 255) {  # setstrutlengths() returned -1
            $valid = 0;
            $save = 0;

            $message = <<EOT;
The ring position computation is not converging. Try another value for ring
centre-to-centre spacing.
EOT

         } else {

            $message = <<EOT;
Front and side views of the frame with the given strut lengths are shown below.
EOT

         }
      }
   }

   if ($save) {
      # save strut length parameters
      $file = "../../config/strutlengths/$frameId.xml";
      open(FILE, ">$file") or die("can't open $file for write");

      $xml = <<EOT;
<strutLengths id="$frameId" strut1Length="$strutLength[0]"
  strut2Length="$strutLength[1]" strut3Length="$strutLength[2]"
  strut4Length="$strutLength[3]" strut5Length="$strutLength[4]"
  strut6Length="$strutLength[5]" ringSpacing="$ringSpacing"/>
EOT

      print(FILE $xml);
      close(FILE) or die("can't open $file for write");
   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Strut Lengths</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Strut Lengths

</div>
<div id="header-nav">
<a href="frameparameters.html">Previous</a><span class="separator"> I </span><a href="xrayparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="algorithms.html">Algorithms</a><span class="separator"> I </span><a href="software.html">Software</a><span class="separator"> I </span><a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a></div>
</div>
<div id="page-contents">
<h1>Strut Lengths</h1>

<div class="page-map">
<dl>
</dl>
</div>

EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }
                                                     
   print <<EOT;
<div class="section">

<p>When frame parameters have been defined, and the frame has been fitted to
the patient, enter the frame identifier, initial strut lengths and approximate
ring spacing here. The strut lengths required are the precise distances between
universal joint centres.</p>

<p>If you enter a frame identifier for which you have previously defined
initial strut lengths and approximate ring spacing, and leave the data fields
empty, then the saved data will be displayed for inspection or editing.</p>

<form action="strutlengths.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="frameId" size="30" type="text" value="$frameId">Frame identifier</p>

</fieldset>

<p>The approximate ring centre-to-centre spacing helps the least-squares search
algorithm to localise the solution to the forward kinematics problem. If in
doubt enter say 70% of the maximum strut length.</p>

<fieldset>

<p><input name="strut1Length" size="6" type="text" value="$strutLength[0]">Strut 1 length</p>

<p><input name="strut2Length" size="6" type="text" value="$strutLength[1]">Strut 2 length</p>

<p><input name="strut3Length" size="6" type="text" value="$strutLength[2]">Strut 3 length</p>

<p><input name="strut4Length" size="6" type="text" value="$strutLength[3]">Strut 4 length</p>

<p><input name="strut5Length" size="6" type="text" value="$strutLength[4]">Strut 5 length</p>

<p><input name="strut6Length" size="6" type="text" value="$strutLength[5]">Strut 6 length</p>

<p><input name="ringSpacing" size="6" type="text" value="$ringSpacing">Approximate ring 
centre-to-centre spacing</p>

</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
</tr>
</table>
</center>

</form>
</div>

EOT

   if ($valid) {

      print <<EOT;
<p><center>
<img src="graphics/frontview$fileSuffix.png"></img>
<img src="graphics/sideview$fileSuffix.png"></img>
</center></p>

EOT

   }

   print <<EOT;
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Strut Lengths

</div>
<div id="footer-nav">
<a href="strutlengths.html">Top</a><span class="separator"> I </span><a href="frameparameters.html">Previous</a><span class="separator"> I </span><a href="xrayparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
