#!/usr/bin/perl -w

# name:     strutparameters.cgi
# function: hexapodfixator cgi script
# location: hexapodfixator/cgi/strutparameters.cgi
# author:   John Storrs
# date:     2/7/07
# version:  1.0


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;


# main
{
   my ($query, $strutParameters, $strutId, $minLength, $maxLength, $diameter,
       $screwRadius, $screwLength, $nutRadius, $nutLength, $tubeRadius,
       $tubeLength, $jointRadius, $jointOffset, $message, $file, $xml,
       $command, $valid, $save);

   # create CGI object
   $query = new CGI;

   # get form data
   $strutId = $query->param('strutId');
   $minLength = $query->param('minLength');
   $maxLength = $query->param('maxLength');
   $diameter = $query->param('diameter');
   $screwRadius = $query->param('screwRadius');
   $screwLength = $query->param('screwLength');
   $nutRadius = $query->param('nutRadius');
   $nutLength = $query->param('nutLength');
   $tubeRadius = $query->param('tubeRadius');
   $tubeLength = $query->param('tubeLength');
   $jointRadius = $query->param('jointRadius');
   $jointOffset = $query->param('jointOffset');

   # validate strut identifier
   $valid = 1;
   $save = 1;
   if (!$strutId || ($strutId =~ m/[^\w\d-]/)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter a valid strut identifier. Permitted characters in identifiers are
letters (A-Za-z), digits (0-9), hyphen (-) and underscore (_).
EOT

   }

   if ($valid) {
      # if no strut parameters entered ...
      if (!$minLength && !$maxLength && !$diameter && 
          !$screwRadius && !$screwLength &&
          !$nutRadius && !$nutLength &&
          !$tubeRadius && !$tubeLength &&
          !$jointRadius && !$jointOffset) {
         $save = 0;

         # if strut definition exists ...
         $file = 
            "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            # get saved strut parameters
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $minLength = $strutParameters->attribute('minLength');
            $maxLength = $strutParameters->attribute('maxLength');
            $diameter = $strutParameters->attribute('diameter');
            $screwRadius = $strutParameters->attribute('screwRadius');
            $screwLength = $strutParameters->attribute('screwLength');
            $nutRadius = $strutParameters->attribute('nutRadius');
            $nutLength = $strutParameters->attribute('nutLength');
            $tubeRadius = $strutParameters->attribute('tubeRadius');
            $tubeLength = $strutParameters->attribute('tubeLength');
            $jointRadius = $strutParameters->attribute('jointRadius');
            $jointOffset = $strutParameters->attribute('jointOffset');

            $message = <<EOT;
Saved parameters for '$strutId' are displayed below. You can edit and save them
again, or use the data for another strut definition.
EOT

         } else {
            # no strut definition exists
            $valid = 0;

            $message = <<EOT;
Strut '$strutId' not found.
EOT

         }
      }
   }

   if ($save) {
      # save strut parameters
      $file = "../../config/strutparameters/$strutId.xml";
      open(FILE, ">$file") or die("can't open $file for write");

      $xml = <<EOT;
<strutParameters id="$strutId" minLength="$minLength" maxLength="$maxLength"
  diameter="$diameter" screwRadius="$screwRadius" screwLength="$screwLength"
  nutRadius="$nutRadius" nutLength="$nutLength" tubeRadius="$tubeRadius"
  tubeLength="$tubeLength" jointRadius="$jointRadius"
  jointOffset="$jointOffset"/>
EOT

      print(FILE $xml);
      close(FILE) or die("can't open $file for write");

      $message = <<EOT;
Parameters for strut '$strutId' have been saved. 
Please print this page as a record.
EOT
   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Strut Parameters</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Strut Parameters

</div>
<div id="header-nav">
<a href="ringparameters.html">Previous</a><span class="separator"> I </span><a href="frameparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="algorithms.html">Algorithms</a><span class="separator"> I </span><a href="software.html">Software</a><span class="separator"> I </span><a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a></div>
</div>
<div id="page-contents">
<h1>Strut Parameters</h1>

<div class="page-map">
<dl>
</dl>
</div>

EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }

   print <<EOT;
<div class="section">

<form action="strutparameters.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<p>Fixator struts are variable length screw devices which are adjustable by
hand. They have a universal joint at each end, and are provided with a means of
length measurement. The universal joints terminate in screw fittings for
attachment to the fixator rings.</p>

<p>Define strut parameters here, with a strut identifier for use when defining
fixator frame parameters. The identifier can be a manufacturer's part number or
other string. Permitted characters are letters (A-Za-z), digits (0-9), hyphen
(-) and underscore (_).</p>

<p>If you enter an existing strut identifier, and do not enter any strut
data, then data for the selected strut will be displayed for inspection or
editing.</p>

<fieldset>

<p><input name="strutId" size="30" type="text" value="$strutId">Strut identifier</p>

</fieldset>

<p>Apart from the minimum and maximum lengths, used to check if strut length
settings are valid, and the joint offset, these parameters only affect
generated graphics, not fixator kinematics. The joint offset defines the
distance from the surface of the frame to the joint centre.</p>

<fieldset>

<p><input name="minLength" size="6" type="text" value="$minLength">Length between
joint centres when strut is fully closed</p>

<p><input name="maxLength" size="6" type="text" value="$maxLength">Length between
joint centres when strut is fully open</p>

<p><input name="diameter" size="6" type="text" value="$diameter">Body diameter
</p>

<p><input name="screwRadius" size="6" type="text" value="$screwRadius">Screw radius
</p>

<p><input name="screwLength" size="6" type="text" value="$screwLength">Screw length
</p>

<p><input name="nutRadius" size="6" type="text" value="$nutRadius">Nut radius
</p>

<p><input name="nutLength" size="6" type="text" value="$nutLength">Nut length
</p>

<p><input name="tubeRadius" size="6" type="text" value="$tubeRadius">Nut tube radius
</p>

<p><input name="tubeLength" size="6" type="text" value="$tubeLength">Nut tube length
</p>

<p><input name="jointRadius" size="6" type="text" value="$jointRadius">Joint radius
</p>

<p><input name="jointOffset" size="6" type="text" value="$jointOffset">Joint offset
</p>

</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
</tr>
</table>
</center>

</form>
</div>
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Strut Parameters

</div>
<div id="footer-nav">
<a href="strutparameters.html">Top</a><span class="separator"> I </span><a href="ringparameters.html">Previous</a><span class="separator"> I </span><a href="frameparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>

EOT

}
