
// Copyright 1996 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#include "primitive.h"


// class Primitive member functions

void 
Primitive::emitPovraySource
  (ofstream &povraySource, const Matrix4)
{
   povraySource << "emitPovraySource - virtual function not defined for primitive\n";
}


// class Cylinder member functions

Cylinder::Cylinder
  (const Vector3 p1, const Vector3 p2, const double rad )
{
   end1 = p1;
   end2 = p2;
   radius = rad;
}

void 
Cylinder::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "cylinder {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius;
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";
}


// class Sphere member functions

Sphere::Sphere
  (const Vector3 p, const double rad )
{
   centre = p;
   radius = rad;
}

void 
Sphere::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "sphere {\n";
   povraySource << "   ";
   centre.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius;
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";
}


// class Box member functions

Box::Box
  (const Vector3 p1, const Vector3 p2)
{
   corner1 = p1;
   corner2 = p2;
}

void 
Box::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "box {\n";
   povraySource << "   ";
   corner1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   corner2.emitPovraySource(povraySource, transformation);
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";
}


// class Cone member functions

Cone::Cone
  (const Vector3 p1, const double rad1,
   const Vector3 p2, const double rad2)
{
   end1 = p1;
   end2 = p2;
   radius1 = rad1;
   radius2 = rad2;
}

void 
Cone::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "cone {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius1;
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius2;
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";
}


// class OpenCylinder member functions

OpenCylinder::OpenCylinder
  (const Vector3 p1, const Vector3 p2, 
   const double outerRadius, const double innerRadius)
{
   end1 = p1;
   end2 = p2;
   this->outerRadius = outerRadius;
   this->innerRadius = innerRadius;
}

void 
OpenCylinder::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "difference {\n";

   povraySource << "cylinder {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << outerRadius;
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";

   povraySource << "cylinder {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << innerRadius;
   povraySource << "\n";
   povraySource << "open\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";

   povraySource << "}\n";
}


// experimental

// class Union member functions

Union::Union
  (Primitive& prim1, Primitive& prim2)
{
   primitive1 = &prim1;
   primitive2 = &prim2;
}

void 
Union::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "union {\n";
   povraySource << "   ";
   primitive1->emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   primitive2->emitPovraySource(povraySource, transformation);
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";
}
