// Copyright 2007 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#ifndef FixatorRingClass
#define FixatorRingClass

#include <fstream>
#include "Vector3.H"
#include "Matrix4.H"

// forward declaration
class FixatorStrut;

const int maxCircles = 4;
const int maxHoles = 100;

struct HoleCircle {
   double radius;
   int holeCount;
};

struct RingParameters {
   double innerRadius;
   double outerRadius; 
   double thickness;
   double jointOffset;
   double jointRadius;
   double supportRadius;
   double boneRadius;
   HoleCircle holeCircle[maxCircles];
   int circleCount;
};

struct Coords {
   Vector3 centreCoords;   // ring centre
   Vector3 ringCoords[6];  // base of joint support
   Vector3 jointCoords[6]; // joint centre
   Vector3 boneCoords[2];  // deformity parameters
};

class FixatorRing
{
   public:
      FixatorRing();
      void setParameters(const int id, const RingParameters parameters);
      void connectStrut(FixatorStrut *strut, 
                        const int circleId, const int Holeid);
      void setLocalBoneCoords(const Vector3 point0, const Vector3 point1);
      void transformTo(const Vector3 rotation, const Vector3 translation);
      void transformBy(const Vector3 rotation, const Vector3 translation);
      void emitPovraySource(ofstream &povraySource, const bool showBone);

      int id;  // 0..1
      RingParameters parameters;
      Vector3 holeCoords[maxCircles][maxHoles];
      Matrix4 transformation;
      Coords local, global;
      FixatorStrut *strut[6];
};

#endif
