// Copyright 2007 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

// FixatorStrut class

#include "primitive.h"
#include "artifact.h"
#include "fixatorstrut.h"
#include "fixatorring.h"


void
FixatorStrut::setParameters(const int id, const StrutParameters parameters)
{
   this->id = id;
   this->parameters = parameters;
}


void
FixatorStrut::connectRings(FixatorRing *ring0, FixatorRing *ring1,
                           const int circleId0, const int holeId0,
                           const int circleId1, const int holeId1)
{
   ring[0] = ring0;
   ring[1] = ring1;
   ring0->connectStrut(this, circleId0, holeId0);
   ring1->connectStrut(this, circleId1, holeId1);
}


double
FixatorStrut::length()
{
   return (ring[0]->global.jointCoords[id].distanceTo
             (ring[1]->global.jointCoords[id]));
}


bool 
FixatorStrut::inRange()
{
   double strutLength = length();
   return ((strutLength >= parameters.minLength) && 
           (strutLength <= parameters.maxLength));
}


void
FixatorStrut::emitPovraySource(ofstream &povraySource)
{
   double strutLength = length();
   Vector3 end0(0.0, 0.0, 0.0);
   Vector3 end1(0.0, 0.0, strutLength);
   Vector3 local, global, translation, rotation;
   Artifact strut("strut");
   Cylinder screw, nut, tube, topJoint, bottomJoint;

   local = end1 - end0;
   global = ring[1]->global.jointCoords[id] - 
               ring[0]->global.jointCoords[id];
   rotation = local.rotationsTo(global);
   translation = ring[0]->global.jointCoords[id];

   bottomJoint = Cylinder
     (Vector3(0.0, 0.0, parameters.clearance),
      Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance),
                           parameters.jointRadius);
   strut.add(&bottomJoint);

   tube = Cylinder
     (Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance),
      Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance + 
                           parameters.tubeLength),
      parameters.tubeRadius);
   strut.add(&tube);

   nut = Cylinder
     (Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance + 
                           parameters.tubeLength),
      Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance + 
                           parameters.tubeLength + parameters.nutLength),
      parameters.nutRadius);
   strut.add(&nut);

   screw = Cylinder
     (Vector3(0.0, 0.0, strutLength - parameters.screwLength - 
                           parameters.jointOffset),
      Vector3(0.0, 0.0, strutLength - parameters.jointOffset),
      parameters.screwRadius);
   strut.add(&screw);

   topJoint = Cylinder
     (Vector3(0.0, 0.0, strutLength - parameters.jointOffset),
      Vector3(0.0, 0.0, strutLength - parameters.clearance),
      parameters.jointRadius);
   strut.add(&topJoint);

   strut.emitPovraySource(povraySource, 
                          Matrix4(identity3D(), rotation, translation));
}
