
// C++ Vector and Matrix Algebra Routines from Graphics Gems IV	
// Author: Jean-Francois DOUE
// Version 3.1 --- October 1993
// Minor modifications by John Storrs

#ifndef Matrix3Class
#define Matrix3Class

#include <iostream>
#include <stdlib.h>

#include "MathsPrefix.H"
#include "Vector2.H"
#include "Vector3.H"

using namespace std;

// 3x3 Matrix Class

class Matrix3
{
protected:

 Vector3 v[3];

public:

// Constructors

Matrix3();
Matrix3(const Vector3& v0, const Vector3& v1, const Vector3& v2);
Matrix3(const double d);
Matrix3(const Matrix3& m);

// Assignment operators

Matrix3& operator = ( const Matrix3& m );  // assignment of a Matrix3
Matrix3& operator += ( const Matrix3& m );  // incrementation by a Matrix3
Matrix3& operator -= ( const Matrix3& m );  // decrementation by a Matrix3
Matrix3& operator *= ( const double d );  // multiplication by a constant
Matrix3& operator /= ( const double d );  // division by a constant
Vector3& operator [] ( int i);  // indexing

// special functions

Matrix3 transpose();  // transpose
Matrix3 inverse();  // inverse
Matrix3& apply(V_FCT_PTR fct);  // apply a function to each element

// friends

friend Matrix3 operator - (const Matrix3& a);  // -m1
friend Matrix3 operator + (const Matrix3& a, const Matrix3& b);	 // m1 + m2
friend Matrix3 operator - (const Matrix3& a, const Matrix3& b);	 // m1 - m2
friend Matrix3 operator * (Matrix3& a, Matrix3& b);  // m1 * m2
friend Matrix3 operator * (const Matrix3& a, const double d);  // m1 * 3.0
friend Matrix3 operator * (const double d, const Matrix3& a);  // 3.0 * m1
friend Matrix3 operator / (const Matrix3& a, const double d);  // m1 / 3.0
friend int operator == (const Matrix3& a, const Matrix3& b);  // m1 == m2 ?
friend int operator != (const Matrix3& a, const Matrix3& b);  // m1 != m2 ?
friend ostream& operator << (ostream& s, Matrix3& m);  // stream output
friend istream& operator >> (istream& s, Matrix3& m);  // stream input
friend void swap(Matrix3& a, Matrix3& b);  // swap m1 & m2

// necessary friend declarations

// linear transforms
friend Vector3 operator * (const Matrix3& a, const Vector3& v);
friend Vector2 operator * (const Matrix3& a, const Vector2& v);
};

// functions

Matrix3 identity2D();  // identity 2D
Matrix3 translation2D(Vector2& v);  // translation 2D
Matrix3 rotation2D(Vector2& Center, const double angle);  // rotation 2D
Matrix3 scaling2D(Vector2& scaleVector);  // scaling 2D

#endif
