
// C++ Vector and Matrix Algebra Routines from Graphics Gems IV
// Author: Jean-Francois DOUE
// Version 3.1 --- October 1993	
// Minor modifications by John Storrs

#ifndef Matrix4Class
#define Matrix4Class

#include <iostream>
#include <stdlib.h>

#include "MathsPrefix.H"
#include "Vector3.H"
#include "Vector4.H"

using namespace std;

// 4x4 Matrix Class

class Matrix4
{
protected:

 Vector4 v[4];

public:

// Constructors

Matrix4();
Matrix4
  (const Vector4& v0, const Vector4& v1, 
   const Vector4& v2, const Vector4& v3);
Matrix4
  (const double d);
Matrix4
  (const Matrix4& m);
Matrix4  // concatenate matrix and transformations
  (const Matrix4& matrix,
   const Vector3& rotation, const Vector3& translation);

// Assignment operators

Matrix4& operator  = ( const Matrix4& m );  // assignment of a Matrix4
Matrix4& operator += ( const Matrix4& m );  // incrementation by a Matrix4
Matrix4& operator -= ( const Matrix4& m );  // decrementation by a Matrix4
Matrix4& operator *= ( const double d );  // multiplication by a constant
Matrix4& operator /= ( const double d );  // division by a constant
Vector4& operator [] ( int i);  // indexing

// special functions

Matrix4 transpose();  // transpose
Matrix4 inverse();  // inverse
Matrix4& apply(V_FCT_PTR fct);  // apply a function to each element

// friends

friend Matrix4 operator - (const Matrix4& a);  // -m1
friend Matrix4 operator + (const Matrix4& a, const Matrix4& b);  // m1 + m2
friend Matrix4 operator - (const Matrix4& a, const Matrix4& b);  // m1 - m2
friend Matrix4 operator * (Matrix4& a, Matrix4& b);  // m1 * m2
friend Matrix4 operator * (const Matrix4& a, const double d);  // m1 * 4.0
friend Matrix4 operator * (const double d, const Matrix4& a);  // 4.0 * m1
friend Matrix4 operator / (const Matrix4& a, const double d);  // m1 / 3.0
friend int operator == (const Matrix4& a, const Matrix4& b);  // m1 == m2 ?
friend int operator != (const Matrix4& a, const Matrix4& b);  // m1 != m2 ?
friend ostream& operator << (ostream& s, Matrix4& m);  // stream output
friend istream& operator >> (istream& s, Matrix4& m);  // stream input
friend void swap(Matrix4& a, Matrix4& b);  // swap m1 & m2

// necessary friend declarations

// linear transforms
friend Vector4 operator * (const Matrix4& a, const Vector4& v);	    
friend Vector3 operator * (const Matrix4& a, const Vector3& v);
};

// functions

Matrix4 identity3D();  // identity 3D
Matrix4 translation3D(const Vector3& v);  // translation 3D
Matrix4 rotation3D(const Vector3& Axis, const double angle);  // rotation 3D
Matrix4 scaling3D(const Vector3& scaleVector);  // scaling 3D
Matrix4 perspective3D(const double d);  // perspective 3D

#endif
