
// C++ Vector and Matrix Algebra Routines from Graphics Gems IV
// Author: Jean-Francois DOUE
// Version 3.1 --- October 1993
// Minor modifications by John Storrs

#ifndef Vector2Class
#define Vector2Class

#include <iostream>
#include <stdlib.h>

#include "MathsPrefix.H"
#include "Vector3.H"
#include "Matrix3.H"

using namespace std;

// 2D Vector Class

class Vector2
{
protected:

 double n[2];

public:

// Constructors

Vector2();
Vector2(const double x, const double y);
Vector2(const double d);
Vector2(const Vector2& v);  // copy constructor
Vector2(const Vector3& v);  // cast v3 to v2
Vector2(const Vector3& v, int dropAxis);  // cast v3 to v2

// Assignment operators

Vector2& operator = ( const Vector2& v );  // assignment of a Vector2
Vector2& operator += ( const Vector2& v );  // incrementation by a Vector2
Vector2& operator -= ( const Vector2& v );  // decrementation by a Vector2
Vector2& operator *= ( const double d );  // multiplication by a constant
Vector2& operator /= ( const double d );  // division by a constant
double& operator [] ( int i);  // indexing

// special functions

double length();  // length of a Vector2
double length2();  // squared length of a Vector2
Vector2& normalize();  // normalize a Vector2
Vector2& apply(V_FCT_PTR fct);  // apply a function to each component

// friends

friend Vector2 operator - (const Vector2& v);  // -v1
friend Vector2 operator + (const Vector2& a, const Vector2& b);  // v1 + v2
friend Vector2 operator - (const Vector2& a, const Vector2& b);  // v1 - v2
friend Vector2 operator * (const Vector2& a, const double d);  // v1 * 3.0
friend Vector2 operator * (const double d, const Vector2& a);  // 3.0 * v1
friend Vector2 operator * (const Matrix3& a, const Vector2& v);  // M . v
friend Vector2 operator * (const Vector2& v, Matrix3& a);  // v . M
friend double operator * (const Vector2& a, const Vector2& b);  // dot product
friend Vector2 operator / (const Vector2& a, const double d);  // v1 / 3.0
friend Vector3 operator ^ (const Vector2& a, const Vector2& b);  // cross product
friend int operator == (const Vector2& a, const Vector2& b);  // v1 == v2 ?
friend int operator != (const Vector2& a, const Vector2& b);  // v1 != v2 ?
friend ostream& operator << (ostream& s, Vector2& v);  // stream output
friend istream& operator >> (istream& s, Vector2& v);  // stream input
friend void swap(Vector2& a, Vector2& b);  // swap v1 & v2
friend Vector2 min(const Vector2& a, const Vector2& b);	 // min(v1, v2)
friend Vector2 max(const Vector2& a, const Vector2& b);  // max(v1, v2)
friend Vector2 prod(const Vector2& a, const Vector2& b);  // term by term

// necessary friend declarations

friend class Vector3;
};

#endif
