
// C++ Vector and Matrix Algebra Routines from Graphics Gems IV
// Author: Jean-Francois DOUE
// Version 3.1 --- October 1993
// Minor modifications by John Storrs

#ifndef Vector3Class
#define Vector3Class

#include <fstream>
#include <stdlib.h>
#include "MathsPrefix.H"

using namespace std;

// 3D Vector Class

class Vector3
{
protected:

 double n[3];

public:

// Constructors

Vector3();
Vector3(const double x, const double y, const double z);
Vector3(const double d);
Vector3(const Vector3& v);  // copy constructor
Vector3(const Vector2& v);  // cast v2 to v3
Vector3(const Vector2& v, double d);  // cast v2 to v3
Vector3(const Vector4& v);  // cast v4 to v3
Vector3(const Vector4& v, int dropAxis);  // cast v4 to v3

// Assignment operators

Vector3& operator  = ( const Vector3& v );  // assignment of a Vector3
Vector3& operator += ( const Vector3& v );  // incrementation by a Vector3
Vector3& operator -= ( const Vector3& v );  // decrementation by a Vector3
Vector3& operator *= ( const double d );  // multiplication by a constant
Vector3& operator /= ( const double d );  // division by a constant
double& operator [] ( int i);  // indexing
double operator [] ( int i) const;  // indexing consts

// special functions

double length();  // length of a Vector3
double length2();  // squared length of a Vector3
Vector3& normalize();  // normalize a Vector3
Vector3& apply(V_FCT_PTR fct);  // apply a function to each component

// JS additions

Vector3 translate(const Vector3 translations);
Vector3 rotate(const Vector3 rotations);
double distanceTo(const Vector3 vector) const;
Vector3 rotationsTo(const Vector3 target) const;
void emitPovraySource
   (ofstream &povraySource, const Matrix4 transformation) const;

// friends

friend Vector3 operator - (const Vector3& v);  // -v1
friend Vector3 operator + (const Vector3& a, const Vector3& b);  // v1 + v2
friend Vector3 operator - (const Vector3& a, const Vector3& b);  // v1 - v2
friend Vector3 operator * (const Vector3& a, const double d);  // v1 * 3.0
friend Vector3 operator * (const double d, const Vector3& a);  // 3.0 * v1
friend Vector3 operator * (const Matrix4& a, const Vector3& v);  // M . v
friend Vector3 operator * (const Vector3& v, Matrix4& a);  // v . M
friend double operator * (const Vector3& a, const Vector3& b);  // dot product
friend Vector3 operator / (const Vector3& a, const double d);  // v1 / 3.0
friend Vector3 operator ^ (const Vector3& a, const Vector3& b);  // cross product
friend int operator == (const Vector3& a, const Vector3& b);  // v1 == v2 ?
friend int operator != (const Vector3& a, const Vector3& b);  // v1 != v2 ?
friend ostream& operator << (ostream& s, Vector3& v);  // stream output
friend istream& operator >> (istream& s, Vector3& v);  // stream input
friend void swap(Vector3& a, Vector3& b);  // swap v1 & v2
friend Vector3 min(const Vector3& a, const Vector3& b);  // min(v1, v2)
friend Vector3 max(const Vector3& a, const Vector3& b);  // max(v1, v2)
friend Vector3 prod(const Vector3& a, const Vector3& b);  // term by term

// necessary friend declarations

friend class Vector2;
friend class Vector4;
friend class Matrix3;
// linear transform
friend Vector2 operator * (const Matrix3& a, const Vector2& v);
// matrix 3 product
friend Matrix3 operator * (Matrix3& a, Matrix3& b);
};

#endif
