
// Copyright 1996 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#include <math.h>
#include <fstream>
#include <sstream>
#include <string>
#include "Vector3.H"
#include "fixatorframe.h"

using namespace std;

int main(int argc, char **argv)
{
   int i, j, status;
   int holeCircleId[2];
   int ringHoleId[2][6];
   double rotationX, rotationY, rotationZ;
   double translationX, translationY, translationZ;
   string fileName, fileSuffix, command;
   RingParameters ringParameters[2];
   StrutParameters strutParameters[6];
   FixatorFrame frame;

   // get ring parameters from command line
   i = 1;
   for (j = 0; j < 2; j++) {
      ringParameters[j].innerRadius = atof(argv[i++]); 
      ringParameters[j].outerRadius =  atof(argv[i++]);
      ringParameters[j].thickness =  atof(argv[i++]);
      if (j == 0) 
         ringParameters[j].jointOffset = 10.2; 
      else
         ringParameters[j].jointOffset = -10.2;
      ringParameters[j].jointRadius = 6.0;
      ringParameters[j].supportRadius = 6.0;
      ringParameters[j].boneRadius = 3.0; 
      ringParameters[j].holeCircle[0].radius =  atof(argv[i++]);
      ringParameters[j].holeCircle[0].holeCount =  atoi(argv[i++]);
      ringParameters[j].holeCircle[1].radius = atof(argv[i++]);
      ringParameters[j].holeCircle[1].holeCount = atoi(argv[i++]);
      ringParameters[j].circleCount = 2;
      frame.ring[j].setParameters(j, ringParameters[j]);
   }

   // get strut parameters from command line
   for (j = 0; j < 6; j++) {
      strutParameters[j].minLength = atof(argv[i++]);
      strutParameters[j].maxLength = atof(argv[i++]);
      strutParameters[j].diameter = atof(argv[i++]);
      strutParameters[j].screwRadius = atof(argv[i++]);
      strutParameters[j].screwLength = atof(argv[i++]);
      strutParameters[j].nutRadius = atof(argv[i++]);
      strutParameters[j].nutLength = atof(argv[i++]);
      strutParameters[j].tubeRadius = atof(argv[i++]);
      strutParameters[j].tubeLength = atof(argv[i++]);
      strutParameters[j].jointRadius = atof(argv[i++]);
      strutParameters[j].jointOffset = atof(argv[i++]);
      strutParameters[j].clearance = 2;
      frame.strut[j].setParameters(j, strutParameters[j]);
   }

   // get strut lengths and approximate ring spacing from command line
   double strutLengths[6];
   double ringSpacing;
   for (j = 0; j < 6; j++) {
      strutLengths[j] = atof(argv[i++]);
   }
   ringSpacing = atof(argv[i++]);

   // get file name suffix from command line
   fileSuffix = argv[i++];

#if 0
   // use 72 hole circle
   frame.strut[0].connectRings
      (&frame.ring[0], &frame.ring[1], 1, 1, 1, 11);
   frame.strut[1].connectRings
      (&frame.ring[0], &frame.ring[1], 1, 23, 1, 13);
   frame.strut[2].connectRings
      (&frame.ring[0], &frame.ring[1], 1, 25, 1, 35);
   frame.strut[3].connectRings
      (&frame.ring[0], &frame.ring[1], 1, 47, 1, 37);
   frame.strut[4].connectRings
      (&frame.ring[0], &frame.ring[1], 1, 49, 1, 59);
   frame.strut[5].connectRings
      (&frame.ring[0], &frame.ring[1], 1, 71, 1, 61);
#endif

   // get hole circle parameters from command line
   for (j = 0; j < 2; j++)
      holeCircleId[j] = atoi(argv[i++]) - 1;  // 1-based -> 0-based
   for (j = 0; j < 6; j++) {
      ringHoleId[0][j] = atoi(argv[i++]) - 1;  // 1-based -> 0-based
      ringHoleId[1][j] = atoi(argv[i++]) - 1;
      frame.strut[j].connectRings(&frame.ring[0], &frame.ring[1], 
                                  holeCircleId[0], ringHoleId[0][j], 
                                  holeCircleId[1], ringHoleId[1][j]);
   }

   status = frame.setStrutLengths(strutLengths, ringSpacing);
   if (status == -1)  // not converging
      return -1;

   // povray output -
   // ensure write directory has drwxrwxrwx permissions
   ostringstream pathName1;
   pathName1 << "graphics/frontview" << fileSuffix << ".pov";
   ofstream povraySource1(pathName1.str().c_str());
   povraySource1 << "#include \"prefix.inc\"" << endl << endl;
   frame.emitPovraySource(povraySource1, false);
   povraySource1.close();
   Vector3 rotation = Vector3(0.0, 0.0, M_PI/2);
   Vector3 translation = Vector3(0.0, 0.0, 0.0);
   frame.transformRingBy(0, rotation, translation); 
   frame.transformRingBy(1, rotation, translation); 
   ostringstream pathName2;
   pathName2 << "graphics/sideview" << fileSuffix << ".pov";
   ofstream povraySource2(pathName2.str().c_str());
   povraySource2 << "#include \"prefix.inc\"" << endl << endl;
   frame.emitPovraySource(povraySource2, false);
   povraySource2.close();

   ostringstream command1;
   command1 << "cd graphics; povray -Ifrontview" << fileSuffix << ".pov";
   command1 << " +A +D +W300 +H300";
   system(command1.str().c_str());
   ostringstream command2;
   command2 << "cd graphics; povray -Isideview" << fileSuffix << ".pov";
   command2 << " +A +D +W300 +H300";
   system(command2.str().c_str());

   return 0;
}
