#!/usr/bin/perl -w

# name:     deformityparameters.cgi
# function: Hexapod fixator cgi script.
# author:   John Storrs, Laboratory for Micro Enterprise.
# license:  Released under the terms of the GNU General Public License.
# date:     $Date: 2008-03-30 19:59:09 +0100 (Sun, 30 Mar 2008) $


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;
use File::Temp qw(mktemp);


# main
{
   my ($query, $valid, $save, $frameId, $file, $xml, $arguments,
       $message, $fileSuffix, $frameParameters, $xrayParameters,
       $deformityParameters, $status, $x1Ring1Frontal,
       $z1Ring1Frontal, $x2Ring1Frontal, $z2Ring1Frontal, $x1Ring2Frontal,
       $z1Ring2Frontal, $x2Ring2Frontal, $z2Ring2Frontal, $x1Ring1Lateral,
       $z1Ring1Lateral, $x2Ring1Lateral, $z2Ring1Lateral, $x1Ring2Lateral,
       $z1Ring2Lateral, $x2Ring2Lateral, $z2Ring2Lateral, $zRotation,
       $ringId, $strutId, $i, $ringParameters, $strutParameters, $strutLengths,
       @ringId, @strutId, @ringHoleId);

   # ensure $PATH includes povray location
   $ENV{PATH} .= ':/usr/local/bin';

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $x1Ring1Frontal = $query->param('x1Ring1Frontal');
   $z1Ring1Frontal = $query->param('z1Ring1Frontal');
   $x2Ring1Frontal = $query->param('x2Ring1Frontal');
   $z2Ring1Frontal = $query->param('z2Ring1Frontal');
   $x1Ring2Frontal = $query->param('x1Ring2Frontal');
   $z1Ring2Frontal = $query->param('z1Ring2Frontal');
   $x2Ring2Frontal = $query->param('x2Ring2Frontal');
   $z2Ring2Frontal = $query->param('z2Ring2Frontal');
   $x1Ring1Lateral = $query->param('x1Ring1Lateral');
   $z1Ring1Lateral = $query->param('z1Ring1Lateral');
   $x2Ring1Lateral = $query->param('x2Ring1Lateral');
   $z2Ring1Lateral = $query->param('z2Ring1Lateral');
   $x1Ring2Lateral = $query->param('x1Ring2Lateral');
   $z1Ring2Lateral = $query->param('z1Ring2Lateral');
   $x2Ring2Lateral = $query->param('x2Ring2Lateral');
   $z2Ring2Lateral = $query->param('z2Ring2Lateral');
   $zRotation = $query->param('zRotation');

   # validate frame identifier
   $valid = 1;
   $save = 1;
   $file = "../../config/frameparameters/$frameId.xml";
   if (!$frameId || !(-f $file)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter an existing frame identifier.
EOT

   } 

   if ($valid) {
      # if no deformity parameters entered ...
      if (!$x1Ring1Frontal && !$z1Ring1Frontal && !$x2Ring1Frontal &&
          !$z2Ring1Frontal && !$x1Ring2Frontal && !$z1Ring2Frontal &&
          !$x2Ring2Frontal && !$z2Ring2Frontal && !$x1Ring1Lateral &&
          !$z1Ring1Lateral && !$x2Ring1Lateral && !$z2Ring1Lateral &&
          !$x1Ring2Lateral && !$z1Ring2Lateral && !$x2Ring2Lateral &&
          !$z2Ring2Lateral && !$zRotation) {
         $save = 0;

         # if deformity parameter definition exists ...
         $file = "../../config/deformityparameters/$frameId.xml";
         if (-f $file) {
            # get deformity parameters
            $xml = importFile($file);
            $deformityParameters = parseXml($xml);
            $x1Ring1Frontal = 
               $deformityParameters->attribute('x1Ring1Frontal');
            $z1Ring1Frontal = 
               $deformityParameters->attribute('z1Ring1Frontal');
            $x2Ring1Frontal = 
               $deformityParameters->attribute('x2Ring1Frontal');
            $z2Ring1Frontal = 
               $deformityParameters->attribute('z2Ring1Frontal');
            $x1Ring2Frontal = 
               $deformityParameters->attribute('x1Ring2Frontal');
            $z1Ring2Frontal = 
               $deformityParameters->attribute('z1Ring2Frontal');
            $x2Ring2Frontal = 
               $deformityParameters->attribute('x2Ring2Frontal');
            $z2Ring2Frontal = 
               $deformityParameters->attribute('z2Ring2Frontal');
            $x1Ring1Lateral = 
               $deformityParameters->attribute('x1Ring1Lateral');
            $z1Ring1Lateral = 
               $deformityParameters->attribute('z1Ring1Lateral');
            $x2Ring1Lateral = 
               $deformityParameters->attribute('x2Ring1Lateral');
            $z2Ring1Lateral = 
               $deformityParameters->attribute('z2Ring1Lateral');
            $x1Ring2Lateral = 
               $deformityParameters->attribute('x1Ring2Lateral');
            $z1Ring2Lateral = 
               $deformityParameters->attribute('z1Ring2Lateral');
            $x2Ring2Lateral = 
               $deformityParameters->attribute('x2Ring2Lateral');
            $z2Ring2Lateral = 
               $deformityParameters->attribute('z2Ring2Lateral');
            $zRotation = 
               $deformityParameters->attribute('zRotation');
         } else {
            $valid = 0;

            $message = <<EOT;
Saved deformity parameters for frame <i>$frameId</i> not found.
EOT

         }
      }
   }

   if ($valid) {
      # get frame parameters
      @ringId = ();
      @strutId = ();
      $file = "../../config/frameparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $frameParameters = parseXml($xml);
         $ringId[0] = $frameParameters->attribute('ring1Id');
         $ringId[1] = $frameParameters->attribute('ring2Id');
         $strutId[0] = $frameParameters->attribute('strut1Id');
         $strutId[1] = $frameParameters->attribute('strut2Id');
         $strutId[2] = $frameParameters->attribute('strut3Id');
         $strutId[3] = $frameParameters->attribute('strut4Id');
         $strutId[4] = $frameParameters->attribute('strut5Id');
         $strutId[5] = $frameParameters->attribute('strut6Id');

         $ringHoleId[0][0] = $frameParameters->attribute('strut1Ring1HoleId');
         $ringHoleId[1][0] = $frameParameters->attribute('strut1Ring2HoleId');
         $ringHoleId[0][1] = $frameParameters->attribute('strut2Ring1HoleId');
         $ringHoleId[1][1] = $frameParameters->attribute('strut2Ring2HoleId');
         $ringHoleId[0][2] = $frameParameters->attribute('strut3Ring1HoleId');
         $ringHoleId[1][2] = $frameParameters->attribute('strut3Ring2HoleId');
         $ringHoleId[0][3] = $frameParameters->attribute('strut4Ring1HoleId');
         $ringHoleId[1][3] = $frameParameters->attribute('strut4Ring2HoleId');
         $ringHoleId[0][4] = $frameParameters->attribute('strut5Ring1HoleId');
         $ringHoleId[1][4] = $frameParameters->attribute('strut5Ring2HoleId');
         $ringHoleId[0][5] = $frameParameters->attribute('strut6Ring1HoleId');
         $ringHoleId[1][5] = $frameParameters->attribute('strut6Ring2HoleId');
      } else {
         $valid = 0;

         $message = <<EOT;
Saved parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   # assemble arguments for call of computebonecoords
   $arguments = '';

   if ($valid) {
      # output ring parameters
      foreach $ringId (@ringId) {
         $file = "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $arguments .= $ringParameters->attribute('innerRadius') . ' ';
            $arguments .= $ringParameters->attribute('outerRadius') . ' ';
            $arguments .= $ringParameters->attribute('thickness') . ' ';
            $arguments .= $ringParameters->attribute('holeCircleRadius') . ' ';
            $arguments .= $ringParameters->attribute('holeCount') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for ring <i>$ringId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output strut parameters
      foreach $strutId (@strutId) {
         $file = "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $arguments .= $strutParameters->attribute('minLength') . ' ';
            $arguments .= $strutParameters->attribute('maxLength') . ' ';
            $arguments .= $strutParameters->attribute('diameter') . ' ';
            $arguments .= $strutParameters->attribute('screwRadius') . ' ';
            $arguments .= $strutParameters->attribute('screwLength') . ' ';
            $arguments .= $strutParameters->attribute('nutRadius') . ' ';
            $arguments .= $strutParameters->attribute('nutLength') . ' ';
            $arguments .= $strutParameters->attribute('tubeRadius') . ' ';
            $arguments .= $strutParameters->attribute('tubeLength') . ' ';
            $arguments .= $strutParameters->attribute('jointRadius') . ' ';
            $arguments .= $strutParameters->attribute('jointOffset') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for strut <i>$strutId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output required strut length parameters
      $file = "../../config/strutlengths/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $strutLengths = parseXml($xml);
         $arguments .= $strutLengths->attribute('ring1Transform') . ' ';
         $arguments .= $strutLengths->attribute('ring2Transform') . ' ';
      } else {
         $valid = 0;

         $message = <<EOT;
Saved strut length parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   if ($valid) {
      # output xray parameters
      $file = "../../config/xrayparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $xrayParameters = parseXml($xml);
         $arguments .= 
            ($xrayParameters->attribute('xrayDistanceFrontal')) . ' ';
         $arguments .= 
            ($xrayParameters->attribute('ring1NearestFrontal') - 1) . ' ';
         $arguments .= 
            ($xrayParameters->attribute('ring2NearestFrontal') - 1) . ' ';
         $arguments .= 
            ($xrayParameters->attribute('xrayDistanceLateral')) . ' ';
         $arguments .= 
            ($xrayParameters->attribute('ring1NearestLateral') - 1) . ' ';
         $arguments .= 
            ($xrayParameters->attribute('ring2NearestLateral') - 1) . ' ';
      } else {
         $valid = 0;

         $message = <<EOT;
Saved x-ray parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   if ($valid) {
      # output required frame parameters
      for $i (0..5) {
         $arguments .= $ringHoleId[0][$i] . ' ';
         $arguments .= $ringHoleId[1][$i] . ' ';
      }

      # output deformity parameters
      $arguments .= $x1Ring1Frontal . ' ';
      $arguments .= $z1Ring1Frontal . ' ';
      $arguments .= $x2Ring1Frontal . ' ';
      $arguments .= $z2Ring1Frontal . ' ';
      $arguments .= $x1Ring2Frontal . ' ';
      $arguments .= $z1Ring2Frontal . ' ';
      $arguments .= $x2Ring2Frontal . ' ';
      $arguments .= $z2Ring2Frontal . ' ';
      $arguments .= $x1Ring1Lateral . ' ';
      $arguments .= $z1Ring1Lateral . ' ';
      $arguments .= $x2Ring1Lateral . ' ';
      $arguments .= $z2Ring1Lateral . ' ';
      $arguments .= $x1Ring2Lateral . ' ';
      $arguments .= $z1Ring2Lateral . ' ';
      $arguments .= $x2Ring2Lateral . ' ';
      $arguments .= $z2Ring2Lateral . ' ';
      $arguments .= $zRotation . ' ';

      # output temporary file name suffix
      $fileSuffix = mktemp('-XXXXXX');
      $arguments .= $fileSuffix . ' ';

      # compute bone coordinates in ring coordinate systems
      $status = system("../../bin/computebonecoords $arguments >> log");  
      if ($status == 255) {  # system() returned -1
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: system call failed.
EOT

      } elsif (!((-f "graphics/frontal$fileSuffix.png") &&
                 (-f "graphics/lateral$fileSuffix.png"))) {
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: generated files not found.
EOT

      }
   }

   if ($valid) {
      $message = <<EOT;
As on the X-Ray Parameters page, full scale
<a href="graphics/frontal-large$fileSuffix.png">frontal</a>
and
<a href="graphics/lateral-large$fileSuffix.png">lateral</a> views of the frame
for printing have been generated, and reduced scale views are shown below
(frontal left, lateral right). Here the deformity parameters are included in
the views in green, for checking.
EOT

   }

   if ($save) {
      if ($frameId ne 'example-frame') {
         # save deformity parameters
         $file = "../../config/deformityparameters/$frameId.xml";
         open(FILE, ">$file") or die("can't open $file for write");

         $xml = <<EOT;
<deformityParameters
  id="$frameId" 
  x1Ring1Frontal="$x1Ring1Frontal"
  z1Ring1Frontal="$z1Ring1Frontal"
  x2Ring1Frontal="$x2Ring1Frontal"
  z2Ring1Frontal="$z2Ring1Frontal"
  x1Ring2Frontal="$x1Ring2Frontal"
  z1Ring2Frontal="$z1Ring2Frontal"
  x2Ring2Frontal="$x2Ring2Frontal"
  z2Ring2Frontal="$z2Ring2Frontal"
  x1Ring1Lateral="$x1Ring1Lateral"
  z1Ring1Lateral="$z1Ring1Lateral"
  x2Ring1Lateral="$x2Ring1Lateral"
  z2Ring1Lateral="$z2Ring1Lateral"
  x1Ring2Lateral="$x1Ring2Lateral"
  z1Ring2Lateral="$z1Ring2Lateral"
  x2Ring2Lateral="$x2Ring2Lateral"
  z2Ring2Lateral="$z2Ring2Lateral"
  zRotation="$zRotation"
/>
EOT

         print(FILE $xml);
         close(FILE) or die("can't open $file for write");
      }
   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Deformity Parameters</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Deformity Parameters

</div>
<div id="header-nav">
<a href="xrayparameters.html">Previous</a><span class="separator"> I </span><a href="deformitycorrection.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a><span class="separator"> I </span><a href="software.html">Software</a></div>
</div>
<div id="page-contents">
<h1>Deformity Parameters</h1>
EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }
                                                     
   print <<EOT;
<div class="section">

<p>Here the bone deformity parameters are recorded. They are measured on the
frontal and lateral x-rays by overlaying them with printouts of the matching
views generated on the previous page. These parameters allow a deformity
correction schedule to be generated.</p>
<p>In the matching views, the blue circles show the x-ray
<i>nearest</i> points, and the red circle the
<i>target</i> point which is also the origin of the 2D coordinate grid. The
grid is behind the frame, in the plane of the x-ray film; the grid spacing is
20mm in each direction. Align the printed and xray images, and use the grid to
capture the bone reference coordinates identified on the x-rays. Enter the
coordinates below. The x axis is across the A3 page, the z axis up it. The y
axis is the line between x-ray source and target point.</p>

<para>If you enter a frame identifier for which you have previously defined
deformity parameters, and leave the data fields empty, then the saved data will
used and can be edited if required. Enter <emphasis>example-frame</emphasis>
for example definitions.</para>

<form action="deformityparameters.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>
<p><input name="frameId" size="30" type="text" value="$frameId">Frame
identifier</p>
</fieldset>

<p>The bone coordinates to be entered here are read off the x-rays with the
help of the generated gridded graphics. The bone segments attached to each
fixator ring are described by initial coordinates X1,Z1 and X2,Z2. After
deformity correction, the X1,Z1 points will be brought into contact and all the
points will lie on a straight line.</p>

<fieldset>
<p>Frontal view:</p>
<div class="inputlist">
<p>Ring 1:
<input name="x1Ring1Frontal" size="4" type="text" value="$x1Ring1Frontal">X1
<input name="z1Ring1Frontal" size="4" type="text" value="$z1Ring1Frontal">Z1
&nbsp;&nbsp;&nbsp;&nbsp;
<input name="x2Ring1Frontal" size="4" type="text" value="$x2Ring1Frontal">X2
<input name="z2Ring1Frontal" size="4" type="text" value="$z2Ring1Frontal">Z2
</p>
<p>Ring 2:
<input name="x1Ring2Frontal" size="4" type="text" value="$x1Ring2Frontal">X1
<input name="z1Ring2Frontal" size="4" type="text" value="$z1Ring2Frontal">Z1
&nbsp;&nbsp;&nbsp;&nbsp;
<input name="x2Ring2Frontal" size="4" type="text" value="$x2Ring2Frontal">X2
<input name="z2Ring2Frontal" size="4" type="text" value="$z2Ring2Frontal">Z2
</p>
</div>
</fieldset>

<fieldset>
<p>Lateral view:</p>
<div class="inputlist">
<p>Ring 1:
<input name="x1Ring1Lateral" size="4" type="text" value="$x1Ring1Lateral">X1
<input name="z1Ring1Lateral" size="4" type="text" value="$z1Ring1Lateral">Z1
&nbsp;&nbsp;&nbsp;&nbsp;
<input name="x2Ring1Lateral" size="4" type="text" value="$x2Ring1Lateral">X2
<input name="z2Ring1Lateral" size="4" type="text" value="$z2Ring1Lateral">Z2
</p>
<p>Ring 2:
<input name="x1Ring2Lateral" size="4" type="text" value="$x1Ring2Lateral">X1
<input name="z1Ring2Lateral" size="4" type="text" value="$z1Ring2Lateral">Z1
&nbsp;&nbsp;&nbsp;&nbsp;
<input name="x2Ring2Lateral" size="4" type="text" value="$x2Ring2Lateral">X2
<input name="z2Ring2Lateral" size="4" type="text" value="$z2Ring2Lateral">Z2
</p>
</div>
</fieldset>

<p>The final deformity parameter which must be provided is the required axial
rotation of the frame about the z axis in degrees. THIS IS NOT YET USED.</p>

<fieldset>
<p>Axial view:</p>
<p><input name="zRotation" size="6" type="text" value="$zRotation">Z rotation
</p>
</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
</tr>
</table>
</center>

</form>
</div>

EOT
   if ($valid) {

      print <<EOT;
<p><center>
<img src="graphics/frontal$fileSuffix.png"></img>
&nbsp; &nbsp; &nbsp;
<img src="graphics/lateral$fileSuffix.png"></img>
</center></p>

EOT

   }

   print <<EOT;
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Deformity Parameters

</div>
<div id="footer-nav">
<a href="deformityparameters.html">Top</a><span class="separator"> I </span><a href="xrayparameters.html">Previous</a><span class="separator"> I </span><a href="deformitycorrection.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
