#!/usr/bin/perl -w

# name:     xrayparameters.cgi
# function: Hexapod fixator cgi script.
# author:   John Storrs, Laboratory for Micro Enterprise.
# license:  Released under the terms of the GNU General Public License.
# date:     $Date: 2008-05-23 22:01:35 +0100 (Fri, 23 May 2008) $


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;
use File::Temp qw(mktemp);


# main
{
   my ($query, $valid, $save, $frameId, $file, $xml, $arguments,
       $message, $ringId, $fileSuffix, $frameParameters, $xrayParameters,
       $ring1NearestFrontal, $ring2NearestFrontal, $ring1NearestLateral,
       $ring2NearestLateral, $xrayDistanceFrontal, $xrayDistanceLateral,
       $strutParameters, $ringParameters, $strutId, $strutLength, $ringSpacing,
       $strutLengths, $status, $i, $j, $line, @ringId, @strutId, @ringHoleId);

   # ensure $PATH includes povray location
   $ENV{PATH} .= ':/usr/local/bin';

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $xrayDistanceFrontal = $query->param('xrayDistanceFrontal');
   $ring1NearestFrontal = $query->param('ring1NearestFrontal');
   $ring2NearestFrontal = $query->param('ring2NearestFrontal');
   $xrayDistanceLateral = $query->param('xrayDistanceLateral');
   $ring1NearestLateral = $query->param('ring1NearestLateral');
   $ring2NearestLateral = $query->param('ring2NearestLateral');

   # validate frame identifier
   $valid = 1;
   $save = 1;
   $file = "../../config/frameparameters/$frameId.xml";
   if (!$frameId || !(-f $file)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter an existing frame identifier.
EOT

   } 

   if ($valid) {
      # if no xray parameters entered ...
      if (!$xrayDistanceFrontal && !$ring1NearestFrontal && 
          !$ring2NearestFrontal && !$xrayDistanceLateral && 
          !$ring1NearestLateral && !$ring2NearestLateral) {
         $save = 0;

         # if xray parameter definition exists ...
         $file = "../../config/xrayparameters/$frameId.xml";
         if (-f $file) {
            # get xray parameters
            $xml = importFile($file);
            $xrayParameters = parseXml($xml);
            $xrayDistanceFrontal = 
               $xrayParameters->attribute('xrayDistanceFrontal');
            $ring1NearestFrontal = 
               $xrayParameters->attribute('ring1NearestFrontal');
            $ring2NearestFrontal = 
               $xrayParameters->attribute('ring2NearestFrontal');
            $xrayDistanceLateral = 
               $xrayParameters->attribute('xrayDistanceLateral');
            $ring1NearestLateral = 
               $xrayParameters->attribute('ring1NearestLateral');
            $ring2NearestLateral = 
               $xrayParameters->attribute('ring2NearestLateral');
         } else {
            $valid = 0;

            $message = <<EOT;
Saved x-ray parameters for frame <i>$frameId</i> not found.
EOT

         }
      }
   }

   if ($valid) {
      # get frame parameters
      @ringId = ();
      @strutId = ();
      $file = "../../config/frameparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $frameParameters = parseXml($xml);
         $ringId[0] = $frameParameters->attribute('ring1Id');
         $ringId[1] = $frameParameters->attribute('ring2Id');
         $strutId[0] = $frameParameters->attribute('strut1Id');
         $strutId[1] = $frameParameters->attribute('strut2Id');
         $strutId[2] = $frameParameters->attribute('strut3Id');
         $strutId[3] = $frameParameters->attribute('strut4Id');
         $strutId[4] = $frameParameters->attribute('strut5Id');
         $strutId[5] = $frameParameters->attribute('strut6Id');

         $ringHoleId[0][0] = $frameParameters->attribute('strut1Ring1HoleId');
         $ringHoleId[1][0] = $frameParameters->attribute('strut1Ring2HoleId');
         $ringHoleId[0][1] = $frameParameters->attribute('strut2Ring1HoleId');
         $ringHoleId[1][1] = $frameParameters->attribute('strut2Ring2HoleId');
         $ringHoleId[0][2] = $frameParameters->attribute('strut3Ring1HoleId');
         $ringHoleId[1][2] = $frameParameters->attribute('strut3Ring2HoleId');
         $ringHoleId[0][3] = $frameParameters->attribute('strut4Ring1HoleId');
         $ringHoleId[1][3] = $frameParameters->attribute('strut4Ring2HoleId');
         $ringHoleId[0][4] = $frameParameters->attribute('strut5Ring1HoleId');
         $ringHoleId[1][4] = $frameParameters->attribute('strut5Ring2HoleId');
         $ringHoleId[0][5] = $frameParameters->attribute('strut6Ring1HoleId');
         $ringHoleId[1][5] = $frameParameters->attribute('strut6Ring2HoleId');
      } else {
         $valid = 0;

         $message = <<EOT;
Saved parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   # assemble arguments for call of transformviews
   $arguments = '';

   if ($valid) {
      # output ring parameters
      foreach $ringId (@ringId) {
         $file = "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $arguments .= $ringParameters->attribute('innerRadius') . ' ';
            $arguments .= $ringParameters->attribute('outerRadius') . ' ';
            $arguments .= $ringParameters->attribute('thickness') . ' ';
            $arguments .= $ringParameters->attribute('holeCircleRadius') . ' ';
            $arguments .= $ringParameters->attribute('holeCount') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for ring <i>$ringId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output strut parameters
      foreach $strutId (@strutId) {
         $file = 
            "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $arguments .= $strutParameters->attribute('minLength') . ' ';
            $arguments .= $strutParameters->attribute('maxLength') . ' ';
            $arguments .= $strutParameters->attribute('diameter') . ' ';
            $arguments .= $strutParameters->attribute('screwRadius') . ' ';
            $arguments .= $strutParameters->attribute('screwLength') . ' ';
            $arguments .= $strutParameters->attribute('nutRadius') . ' ';
            $arguments .= $strutParameters->attribute('nutLength') . ' ';
            $arguments .= $strutParameters->attribute('tubeRadius') . ' ';
            $arguments .= $strutParameters->attribute('tubeLength') . ' ';
            $arguments .= $strutParameters->attribute('jointRadius') . ' ';
            $arguments .= $strutParameters->attribute('jointOffset') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for strut <i>$strutId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output required strut length parameters
      $file = "../../config/strutlengths/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $strutLengths = parseXml($xml);
         $arguments .= $strutLengths->attribute('ring1Transform') . ' ';
         $arguments .= $strutLengths->attribute('ring2Transform') . ' ';
      } else {
         $valid = 0;

         $message = <<EOT;
Saved strut length parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   if ($valid) {
      # output required frame parameters
      for $i (0..5) {
         $arguments .= $ringHoleId[0][$i] . ' ';
         $arguments .= $ringHoleId[1][$i] . ' ';
      }

      # output xray parameters
      $arguments .= $xrayDistanceFrontal . ' ';
      $arguments .= ($ring1NearestFrontal - 1) . ' ';  # 1-based -> 0-based
      $arguments .= ($ring2NearestFrontal - 1) . ' ';
      $arguments .= $xrayDistanceLateral . ' ';
      $arguments .= ($ring1NearestLateral - 1) . ' ';
      $arguments .= ($ring2NearestLateral - 1) . ' ';

      # get temporary file name suffix
      $fileSuffix = mktemp('-XXXXXX');
      $arguments .= $fileSuffix . ' ';

      # generate graphics
      $status = system("../../bin/transformviews $arguments >> log");  
      if ($status == 255) {  # system() returned -1
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: system call failed.
EOT

      } elsif (!((-f "graphics/frontal$fileSuffix.png") &&
                 (-f "graphics/lateral$fileSuffix.png"))) {
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: generated files not found.
EOT

      }
   }

   if ($valid) {
      $message = <<EOT;
Full scale
<a href="graphics/frontal-large$fileSuffix.png">frontal</a>
and 
<a href="graphics/lateral-large$fileSuffix.png">lateral</a> views of the frame
for printing have been generated, and reduced scale views are shown below
(frontal left, lateral right). If the input data are correct, when printed to
scale they should match the x-ray images. The image resolution is 150ppi, and
the full scale size is 280x380mm.  To print on a Linux system using CUPS, save
the full scale images to files by right-clicking on the links above and do
<pre>
lpr -P&lt;printer name&gt; -o ppi=150 &lt;image file name&gt;
</pre>
If the printable area is smaller than the full scale size, CUPS will tile the
images on several sheets for pasting. An A3 or larger printer should print them
to scale on single sheets. Check the scale by measuring the red lines - they
should both be exactly 280mm long. The blue circles show the 'nearest' points,
and the red circle the target point which is also the origin of the coordinate
grid. The grid is in the centre of the frame, where the grid spacing is 20mm in
each direction.  Because it is projected onto the xray film plane, the spacing
in the image is larger than 20mm. Align the printed and xray images, and use
the grid to capture the bone reference coordinates. Enter the coordinates on
the Deformity Parameters page which follows. The x axis is across the A3 page,
the y axis (z actually, but that's a technical detail) up it. 
EOT

   }

   if ($save) {
      if ($frameId ne 'example-frame') {
         # save xray parameters
         $file = "../../config/xrayparameters/$frameId.xml";
         open(FILE, ">$file") or die("can't open $file for write");

         $xml = <<EOT;
<xrayParameters
  id="$frameId" 
  xrayDistanceFrontal="$xrayDistanceFrontal"
  ring1NearestFrontal="$ring1NearestFrontal" 
  ring2NearestFrontal="$ring2NearestFrontal"
  xrayDistanceLateral="$xrayDistanceLateral"
  ring1NearestLateral="$ring1NearestLateral" 
  ring2NearestLateral="$ring2NearestLateral"
/>
EOT

         print(FILE $xml);
         close(FILE) or die("can't open $file for write");
      }
   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>X-Ray Parameters</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>X-Ray Parameters

</div>
<div id="header-nav">
<a href="strutlengths.html">Previous</a><span class="separator"> I </span><a href="deformityparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a><span class="separator"> I </span><a href="software.html">Software</a>
</div>
</div>
<div id="page-contents">
<h1>X-Ray Parameters</h1>
EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }
                                                     
   print <<EOT;
<div class="section">

<p>While taking the initial frontal and lateral x-rays of the patient and
fitted frame, some measurements should be made and recorded here. The software 
will then generate matching views of the frame with a coordinate grid,
for use in determining the deformity parameters. Note that this may take
some time.</p>

<p>If you enter a frame identifier for which you have previously defined x-ray
parameters, and leave the data fields empty, then the saved data will be used
and can be edited if required. Enter <emphasis>example-frame</emphasis> for
example definitions.</p>

<form action="xrayparameters.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="frameId" size="30" type="text" value="$frameId">Frame identifier</p>

</fieldset>

<p>The distance from the x-ray source to the film cartridge defines the
image projection. Numbers identifying the point on each ring nearest to the
x-ray source define the frame orientation. The fixing holes provide a
convenient scale; for example, if a point midway between hole 11 and hole 12 is
nearest the x-ray source, enter 11.5. The software assumes that both rings are
in contact with the film cartridge, and that the x-ray target point is the
middle of the line joining these <i>nearest points</i>.</p>

<fieldset>

<p>Frontal view:</p>

<p><input name="xrayDistanceFrontal" size="6" type="text" 
value="$xrayDistanceFrontal">Distance from x-ray source to film cartridge</p>

<p><input name="ring1NearestFrontal" size="6" type="text"
value="$ring1NearestFrontal">Number identifying nearest point to x-ray source
on ring 1</p>
                                                            
<p><input name="ring2NearestFrontal" size="6" type="text" 
value="$ring2NearestFrontal">Number identifying nearest point to x-ray source
on ring 2</p>

</fieldset>

<fieldset>

<p>Lateral view:</p>

<p><input name="xrayDistanceLateral" size="6" type="text" 
value="$xrayDistanceLateral">Distance from x-ray source to film cartridge</p>

<p><input name="ring1NearestLateral" size="6" type="text" 
value="$ring1NearestLateral">Number identifying nearest point to x-ray source
on ring 1</p>

<p><input name="ring2NearestLateral" size="6" type="text" 
value="$ring2NearestLateral">Number identifying nearest point to x-ray source
on ring 2</p>

</fieldset>

<center>
<table>
<tr>
<td><button type="submit">Submit</button></td>
</tr>
</table>
</center>

</form>
</div>

EOT

   if ($valid) {

      print <<EOT;
<p><center>
<img src="graphics/frontal$fileSuffix.png"></img>
&nbsp; &nbsp; &nbsp;
<img src="graphics/lateral$fileSuffix.png"></img>
</center></p>

EOT

   }

   print <<EOT;
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>X-Ray Parameters

</div>
<div id="footer-nav">
<a href="xrayparameters.html">Top</a><span class="separator"> I </span><a href="strutlengths.html">Previous</a><span class="separator"> I </span><a href="deformityparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
