// Copyright 2007 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

// FixatorStrut class

#include "primitive.h"
#include "artifact.h"
#include "fixatorstrut.h"
#include "fixatorring.h"


void
FixatorStrut::setParameters(const int id, const StrutParameters &parameters)
{
   this->id = id;
   this->parameters = parameters;
}


void
FixatorStrut::connectRings(FixatorRing *ring0, FixatorRing *ring1,
                           const int holeId0, const int holeId1)
{
   ring[0] = ring0;
   ring[1] = ring1;
   ring0->connectStrut(this, holeId0);
   ring1->connectStrut(this, holeId1);
}


double
FixatorStrut::length()
{
   return (ring[0]->global.jointCoords[id].distanceTo
             (ring[1]->global.jointCoords[id]));
}


bool 
FixatorStrut::inRange()
{
   double strutLength = length();
   return ((strutLength >= parameters.minLength) && 
           (strutLength <= parameters.maxLength));
}


void
FixatorStrut::emitPovraySource(ofstream &povraySource)
{
   double strutLength = length();
   Vector3 end0(0.0, 0.0, 0.0);
   Vector3 end1(0.0, 0.0, strutLength);
   Vector3 local, global, translation, rotation;
   Matrix4 transformation;
   Artifact strut("FixatorStrut");
   Cylinder screw, nut, tube, topJoint, bottomJoint;

   local = end1 - end0;
   global = ring[1]->global.jointCoords[id] - 
               ring[0]->global.jointCoords[id];
   transformation = local.alignWith(global);
   translation = ring[0]->global.jointCoords[id] - end0;

   bottomJoint = Cylinder
     (Vector3(0.0, 0.0, parameters.clearance),
      Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance),
      parameters.jointRadius, "FixatorStrut");
   strut.add(&bottomJoint);

   tube = Cylinder
     (Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance),
      Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance + 
         parameters.tubeLength),
      parameters.tubeRadius, "FixatorStrut");
   strut.add(&tube);

   nut = Cylinder
     (Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance + 
         parameters.tubeLength),
      Vector3(0.0, 0.0, parameters.jointOffset - parameters.clearance + 
         parameters.tubeLength + parameters.nutLength),
      parameters.nutRadius, "FixatorStrut");
   strut.add(&nut);

   screw = Cylinder
     (Vector3(0.0, 0.0, strutLength - parameters.screwLength - 
         parameters.jointOffset),
      Vector3(0.0, 0.0, strutLength - parameters.jointOffset),
      parameters.screwRadius, "FixatorStrut");
   strut.add(&screw);

   topJoint = Cylinder
     (Vector3(0.0, 0.0, strutLength - parameters.jointOffset),
      Vector3(0.0, 0.0, strutLength - parameters.clearance),
      parameters.jointRadius, "FixatorStrut");
   strut.add(&topJoint);

   strut.emitPovraySource
      (povraySource, Matrix4(transformation, Vector3(0,0,0), translation));
}
