
// C++ Vector and Matrix Algebra Routines from Graphics Gems IV
// Author: Jean-Francois DOUE
// Version 3.1 --- October 1993
// Minor modifications by John Storrs

#ifndef Vector4Class
#define Vector4Class

#include <iostream>
#include <stdlib.h>
#include "MathsPrefix.H"

using namespace std;

// 4D Vector Class

class Vector4
{
protected:

 double n[4];

public:

// Constructors

Vector4();
Vector4(const double x, const double y, const double z, const double w);
Vector4(const double d);
Vector4(const Vector4& v);  // copy constructor
Vector4(const Vector3& v);  // cast Vector3 to Vector4
Vector4(const Vector3& v, const double d);  // cast Vector3 to Vector4

// Assignment operators

Vector4& operator = ( const Vector4& v );  // assignment of a Vector4
Vector4& operator += ( const Vector4& v );  // incrementation by a Vector4
Vector4& operator -= ( const Vector4& v );  // decrementation by a Vector4
Vector4& operator *= ( const double d );  // multiplication by a constant
Vector4& operator /= ( const double d );  // division by a constant
double& operator [] ( int i);  // indexing

// special functions

double length();  // length of a Vector4
double length2();  // squared length of a Vector4
Vector4& normalize();  // normalize a Vector4
Vector4& apply(V_FCT_PTR fct);  // apply a function to each component

// friends

friend Vector4 operator - (const Vector4& v);  // -v1
friend Vector4 operator + (const Vector4& a, const Vector4& b);  // v1 + v2
friend Vector4 operator - (const Vector4& a, const Vector4& b);  // v1 - v2
friend Vector4 operator * (const Vector4& a, const double d);  // v1 * 3.0
friend Vector4 operator * (const double d, const Vector4& a);  // 3.0 * v1
friend Vector4 operator * (const Matrix4& a, const Vector4& v);  // M . v
friend Vector4 operator * (const Vector4& v, Matrix4& a);  // v . M
friend double operator * (const Vector4& a, const Vector4& b);  // dot product
friend Vector4 operator / (const Vector4& a, const double d);  // v1 / 3.0
friend int operator == (const Vector4& a, const Vector4& b);  // v1 == v2 ?
friend int operator != (const Vector4& a, const Vector4& b);  // v1 != v2 ?
friend ostream& operator << (ostream& s, Vector4& v);  // stream output
friend istream& operator >> (istream& s, Vector4& v);  // stream input
friend void swap(Vector4& a, Vector4& b);  // swap v1 & v2
friend Vector4 min(const Vector4& a, const Vector4& b);  // min(v1, v2)
friend Vector4 max(const Vector4& a, const Vector4& b);  // max(v1, v2)
friend Vector4 prod(const Vector4& a, const Vector4& b);  // term by term

// necessary friend declarations

friend class Vector3;
friend class Matrix4;
// linear transform
friend Vector3 operator * (const Matrix4& a, const Vector3& v);
// matrix 4 product
friend Matrix4 operator * (Matrix4& a, Matrix4& b);
};

#endif
