#!/usr/bin/perl -w

# name:     deformitycorrection.cgi
# function: Hexapod fixator cgi script.
# author:   John Storrs, Laboratory for Micro Enterprise.
# license:  Released under the terms of the GNU General Public License.
# date:     $Date: 2008-06-26 22:40:29 +0100 (Thu, 26 Jun 2008) $


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;
use File::Temp qw(mktemp);


# main
{
   my ($query, $message, $file, $xml, $command, $valid, $save, $frameId, $i,
       $fileSuffix, $correctionSteps, $correctionSchedule, $ringParameters,
       $strutParameters, $frameParameters, $strutLengths, $xrayParameters,
       $deformityParameters, $arguments, $ringId, $strutId, $status, 
       $corrections, $correction, $id, $step, $strut, $stepSize, $strutLength,
       $strutStatus, $table, @steps, @struts, @ringId, @strutId, @ringHoleId);

   # ensure $PATH includes povray location
   $ENV{PATH} .= ':/usr/local/bin';

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $correctionSteps = $query->param('correctionSteps');

   # validate frame identifier
   $valid = 1;
   $save = 1;
   $file = "../../config/frameparameters/$frameId.xml";
   if (!$frameId || !(-f $file)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter an existing frame identifier.
EOT

   }

   if ($valid) {
      # if no correction parameter entered ...
      if (!$correctionSteps) {
         $save = 0;

         # if correction parameter definition exists ...
         $file = "../../config/deformitycorrection/$frameId.xml";
         if (-f $file) {
            # get correction parameter
            $xml = importFile($file);
            $correctionSchedule = parseXml($xml);
            $correctionSteps 
               = $correctionSchedule->attribute('correctionSteps');
         } else {
            $valid = 0;

            $message = <<EOT;
Saved correction parameter for frame <i>$frameId</i> not found.
EOT

         }
      }
   }

   if ($valid) {
      # if any parameters are undefined ...
      if ($correctionSteps eq '') {
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Please define all parameters.
EOT

      }
   }

   if ($valid) {
      # get frame parameters
      @ringId = ();
      @strutId = ();
      $file = "../../config/frameparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $frameParameters = parseXml($xml);
         $ringId[0] = $frameParameters->attribute('ring1Id');
         $ringId[1] = $frameParameters->attribute('ring2Id');
         $strutId[0] = $frameParameters->attribute('strut1Id');
         $strutId[1] = $frameParameters->attribute('strut2Id');
         $strutId[2] = $frameParameters->attribute('strut3Id');
         $strutId[3] = $frameParameters->attribute('strut4Id');
         $strutId[4] = $frameParameters->attribute('strut5Id');
         $strutId[5] = $frameParameters->attribute('strut6Id');

         $ringHoleId[0][0] = $frameParameters->attribute('strut1Ring1HoleId');
         $ringHoleId[1][0] = $frameParameters->attribute('strut1Ring2HoleId');
         $ringHoleId[0][1] = $frameParameters->attribute('strut2Ring1HoleId');
         $ringHoleId[1][1] = $frameParameters->attribute('strut2Ring2HoleId');
         $ringHoleId[0][2] = $frameParameters->attribute('strut3Ring1HoleId');
         $ringHoleId[1][2] = $frameParameters->attribute('strut3Ring2HoleId');
         $ringHoleId[0][3] = $frameParameters->attribute('strut4Ring1HoleId');
         $ringHoleId[1][3] = $frameParameters->attribute('strut4Ring2HoleId');
         $ringHoleId[0][4] = $frameParameters->attribute('strut5Ring1HoleId');
         $ringHoleId[1][4] = $frameParameters->attribute('strut5Ring2HoleId');
         $ringHoleId[0][5] = $frameParameters->attribute('strut6Ring1HoleId');
         $ringHoleId[1][5] = $frameParameters->attribute('strut6Ring2HoleId');
      } else {
         $valid = 0;

         $message = <<EOT;
Saved parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   # assemble arguments for call of computecorrections
   $arguments = "$frameId ";

   if ($valid) {
      # output ring parameters
      foreach $ringId (@ringId) {
         $file = "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $arguments .= $ringParameters->attribute('innerRadius') . ' ';
            $arguments .= $ringParameters->attribute('outerRadius') . ' ';
            $arguments .= $ringParameters->attribute('thickness') . ' ';
            $arguments .= $ringParameters->attribute('holeCircleRadius') . ' ';
            $arguments .= $ringParameters->attribute('holeCount') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for ring <i>$ringId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output strut parameters
      foreach $strutId (@strutId) {
         $file = "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $arguments .= $strutParameters->attribute('minLength') . ' ';
            $arguments .= $strutParameters->attribute('maxLength') . ' ';
            $arguments .= $strutParameters->attribute('screwRadius') . ' ';
            $arguments .= $strutParameters->attribute('screwLength') . ' ';
            $arguments .= $strutParameters->attribute('nutRadius') . ' ';
            $arguments .= $strutParameters->attribute('nutLength') . ' ';
            $arguments .= $strutParameters->attribute('tubeRadius') . ' ';
            $arguments .= $strutParameters->attribute('tubeLength') . ' ';
            $arguments .= $strutParameters->attribute('jointRadius') . ' ';
            $arguments .= $strutParameters->attribute('jointOffset') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for strut <i>$strutId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output required strut length parameters
      $file = "../../config/strutlengths/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $strutLengths = parseXml($xml);
         $arguments .= $strutLengths->attribute('ring1Transform') . ' ';
         $arguments .= $strutLengths->attribute('ring2Transform') . ' ';
      } else {
         $valid = 0;

         $message = <<EOT;
Saved strut length parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   if ($valid) {
      # output xray parameters
      $file = "../../config/xrayparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $xrayParameters = parseXml($xml);
         $arguments .=
            ($xrayParameters->attribute('xrayDistanceFrontal')) . ' ';
         $arguments .=
            ($xrayParameters->attribute('ring1NearestFrontal') - 1) . ' ';
         $arguments .=
            ($xrayParameters->attribute('ring2NearestFrontal') - 1) . ' ';
         $arguments .=
            ($xrayParameters->attribute('xrayDistanceLateral')) . ' ';
         $arguments .=
            ($xrayParameters->attribute('ring1NearestLateral') - 1) . ' ';
         $arguments .=
            ($xrayParameters->attribute('ring2NearestLateral') - 1) . ' ';
      } else {
         $valid = 0;

         $message = <<EOT;
Saved x-ray parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   if ($valid) {
      # output deformity parameters
      $file = "../../config/deformityparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $deformityParameters = parseXml($xml);
         $arguments .= 
            $deformityParameters->attribute('x1Ring1Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z1Ring1Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x2Ring1Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z2Ring1Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x1Ring2Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z1Ring2Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x2Ring2Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z2Ring2Frontal') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x1Ring1Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z1Ring1Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x2Ring1Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z2Ring1Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x1Ring2Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z1Ring2Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('x2Ring2Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('z2Ring2Lateral') . ' ';
         $arguments .= 
            $deformityParameters->attribute('axialExtension') . ' ';
         $arguments .= 
            $deformityParameters->attribute('axialRotation')*3.14159/180 . ' ';
      } else {
         $valid = 0;

         $message = <<EOT;
Saved deformity parameters for frame <i>$frameId</i> not found.
EOT

      }
   }

   if ($valid) {
      # output required frame parameters
      for $i (0..5) {
         $arguments .= $ringHoleId[0][$i] . ' ';
         $arguments .= $ringHoleId[1][$i] . ' ';
      }

      # output correction parameter
      $arguments .= $correctionSteps . ' ';

      # output temporary file name suffix
      $fileSuffix = mktemp('-XXXXXX');
      $arguments .= $fileSuffix . ' ';

      # compute corrections
      $command = "../../bin/computecorrections $arguments >> log$fileSuffix";
      $status = system($command);
      if ($status == 255) {  # system() returned -1
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: system call failed.
EOT

      } elsif (!((-f "graphics/fixator0$fileSuffix.png") ||
            (-f "graphics/fixator1$fileSuffix.png") ||
            (-f "graphics/fixator2$fileSuffix.png") ||
            (-f "graphics/fixator3$fileSuffix.png") ||
            (-f "graphics/fixator4$fileSuffix.png") ||
            (-f "graphics/fixator5$fileSuffix.png") ||
            (-f "corrections$fileSuffix.xml"))) {
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: generated files not found.
EOT

      }
   }

   if ($valid) {

      $message = <<EOT;
The generated correction schedule, and snapshots (frontal left, lateral right)
of the deformity correction process are shown below. In the schedule below, the
step size is the relative motion of the bone segments along the correction path
with each step. The motion is linear, and so constant throughout the process.
If a required strut length is shorter than the minimum allowed, it is shown
in blue. If it is longer than the maximum allowed, it is shown in red.
EOT

      $file = "corrections$fileSuffix.xml";
      $xml = importFile($file);
      $corrections = parseXml($xml);
      @steps = $corrections->nodeList('step');


      $table .= <<EOT;
<table border="1" valign="top" cellpadding="5" cellspacing="0" 
  width="60%" id="corrections">
  <tr>
    <td width="10%">step id</td>
    <td width="12%">step size</td>
    <td width="12%">strut1 length</td>
    <td width="12%">strut2 length</td>
    <td width="12%">strut3 length</td>
    <td width="12%">strut4 length</td>
    <td width="12%">strut5 length</td>
    <td width="12%">strut6 length</td>
  </tr>
EOT

      foreach $step (@steps) {
         $id = $step->attribute('id');
         $stepSize = $step->attribute('stepSize');

         $table .= <<EOT;
  <tr>
    <td width="10%">$id</td>
    <td width="10%">$stepSize</td>
EOT

         @struts = $step->nodeList('strut');
         foreach $strut (@struts) {
            $strutId = $strut->attribute('id');
            $strutLength = $strut->attribute('length');
            $strutStatus = $strut->attribute('status');

            if ($strutStatus eq 'tooLong') {
            
            $table .= <<EOT;
    <td width="12%"><span class="tooLong">$strutLength</span></td>
EOT

            } elsif ($strutStatus eq 'tooShort') {
            
            $table .= <<EOT;
    <td width="12%"><span class="tooShort">$strutLength</span></td>
EOT

            } else {
            
            $table .= <<EOT;
    <td width="12%">$strutLength</td>
EOT

            }
         }

         $table .= <<EOT;
  </tr>

EOT

      }

      $table .= <<EOT;
</table>

EOT

   }

   if ($save) {
      if ($frameId ne 'example-frame') {
         # save correction parameter
         $file = 
            "../../config/deformitycorrection/$frameId.xml";
         open(FILE, ">$file") or die("can't open $file for write");

         $xml = <<EOT;
<correctionSchedule 
  id="$frameId" correctionSteps="$correctionSteps"
/>
EOT

         print(FILE $xml);
         close(FILE) or die("can't open $file for write");
      }
   }
  
   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Deformity Correction</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Deformity Correction

</div>
<div id="header-nav">
<a href="deformityparameters.html">Previous</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a><span class="separator"> I </span><a href="software.html">Software</a></div>
</div>
<div id="page-contents">
<h1>Deformity Correction</h1>
EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }

   print <<EOT;
<div class="section">

<p>The deformity is corrected by adjusting the struts incrementally to bring
the reference bone segments into the required alignment. Enter the required
number of correction steps here. The software will then generate a correction
schedule and illustrative graphics.</p>

<p>If you enter a frame identifier for which you have previously defined
deformity correction parameters, and leave the data fields empty, then the
saved data will be used and can be edited if required. Enter
<i>example-frame</i> for example definitions.</p>

<!-- embedded html -->
<form action="deformitycorrection.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="frameId" size="30" type="text" value="$frameId">Frame 
identifier</p>

</fieldset>

<fieldset>

<p><input name="correctionSteps" size="8" type="text" value="$correctionSteps">
Number of correction steps</p>

</fieldset>

<center><button type="submit">Submit</button></center>

EOT

   if ($valid) {

      print("<br/><br/><center>$table</center>");

      print <<EOT;
<p><center>
<img src="graphics/fixator0$fileSuffix.png"></img>
<img src="graphics/fixator1$fileSuffix.png"></img>
</center></p>
<p><center>
<img src="graphics/fixator2$fileSuffix.png"></img>
<img src="graphics/fixator3$fileSuffix.png"></img>
</center></p>
<p><center>
<img src="graphics/fixator4$fileSuffix.png"></img>
<img src="graphics/fixator5$fileSuffix.png"></img>
</center></p>
<p><center>
<img src="graphics/fixator6$fileSuffix.png"></img>
<img src="graphics/fixator7$fileSuffix.png"></img>
</center></p>

EOT

   }

   print <<EOT;
</form><!-- end embedded html -->
</div>
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Deformity Correction

</div>
<div id="footer-nav">
<a href="deformitycorrection.html">Top</a><span class="separator"> I </span><a href="deformityparameters.html">Previous</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
