#!/usr/bin/perl -w

# name:     strutlengths.cgi
# function: Hexapod fixator cgi script.
# author:   John Storrs, Laboratory for Micro Enterprise.
# license:  Released under the terms of the GNU General Public License.
# date:     $Date: 2008-06-26 22:40:29 +0100 (Thu, 26 Jun 2008) $


use strict;
use CGI;
use lib "$ENV{PXML_HOME}/lib";
use LME::XmlParser;
use LME::XmlNode;
use File::Temp qw(mktemp);


# main
{
   my ($query, $valid, $save, $frameId, $ringSpacing, $file, $xml, $arguments,
       $message, $ringId, $fileSuffix, $strutId, $frameParameters, 
       $ringParameters, $strutParameters, $strutLength, $minLength, $maxLength,
       $command, $status, $i, $j, $line, @ringId, @strutId, @strutLength,
       @ringHoleId, @transformations);

   # ensure $PATH includes povray location
   $ENV{PATH} .= ':/usr/local/bin';

   # create CGI object
   $query = new CGI;

   # get form data
   $frameId = $query->param('frameId');
   $strutLength[0] = $query->param('strut1Length');
   $strutLength[1] = $query->param('strut2Length');
   $strutLength[2] = $query->param('strut3Length');
   $strutLength[3] = $query->param('strut4Length');
   $strutLength[4] = $query->param('strut5Length');
   $strutLength[5] = $query->param('strut6Length');
   $ringSpacing = $query->param('ringSpacing');


   # validate frame identifier
   $valid = 1;
   $save = 1;
   $file = "../../config/frameparameters/$frameId.xml";
   if (!$frameId || !(-f $file)) {
      $valid = 0;
      $save = 0;

      $message = <<EOT;
Please enter an existing frame identifier.
EOT

   } 

   if ($valid) {
      # if no strut length parameters entered ...
      if (!$strutLength[0] && !$strutLength[1] && !$strutLength[2] &&
          !$strutLength[3] && !$strutLength[4] && !$strutLength[5] &&
          !$ringSpacing) {
         $save = 0;

         # if strut length parameter definition exists ...
         $file = "../../config/strutlengths/$frameId.xml";
         if (-f $file) {
            # get strut length parameters
            $xml = importFile($file);
            $frameParameters = parseXml($xml);
            $strutLength[0] = $frameParameters->attribute('strut1Length');
            $strutLength[1] = $frameParameters->attribute('strut2Length');
            $strutLength[2] = $frameParameters->attribute('strut3Length');
            $strutLength[3] = $frameParameters->attribute('strut4Length');
            $strutLength[4] = $frameParameters->attribute('strut5Length');
            $strutLength[5] = $frameParameters->attribute('strut6Length');
            $ringSpacing = $frameParameters->attribute('ringSpacing');
         } else {
            $valid = 0;

    $message = <<EOT;
Saved strut length parameters for frame <i>$frameId</i> not found.
EOT

         }
      }
   }

   if ($valid) {
      # if any parameters are undefined ...
      if (($strutLength[0] eq '') || ($strutLength[1] eq '') || 
          ($strutLength[2] eq '') || ($strutLength[3] eq '') || 
          ($strutLength[4] eq '') || ($strutLength[5] eq '') ||
          ($ringSpacing eq '')) {
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Please define all parameters.
EOT

      }
   }

   if ($valid) {
      # get frame parameters
      @ringId = ();
      @strutId = ();
      $file = "../../config/frameparameters/$frameId.xml";
      if (-f $file) {
         $xml = importFile($file);
         $frameParameters = parseXml($xml);
         $ringId[0] = $frameParameters->attribute('ring1Id');
         $ringId[1] = $frameParameters->attribute('ring2Id');
         $strutId[0] = $frameParameters->attribute('strut1Id');
         $strutId[1] = $frameParameters->attribute('strut2Id');
         $strutId[2] = $frameParameters->attribute('strut3Id');
         $strutId[3] = $frameParameters->attribute('strut4Id');
         $strutId[4] = $frameParameters->attribute('strut5Id');
         $strutId[5] = $frameParameters->attribute('strut6Id');

         $ringHoleId[0][0] = $frameParameters->attribute('strut1Ring1HoleId');
         $ringHoleId[1][0] = $frameParameters->attribute('strut1Ring2HoleId');
         $ringHoleId[0][1] = $frameParameters->attribute('strut2Ring1HoleId');
         $ringHoleId[1][1] = $frameParameters->attribute('strut2Ring2HoleId');
         $ringHoleId[0][2] = $frameParameters->attribute('strut3Ring1HoleId');
         $ringHoleId[1][2] = $frameParameters->attribute('strut3Ring2HoleId');
         $ringHoleId[0][3] = $frameParameters->attribute('strut4Ring1HoleId');
         $ringHoleId[1][3] = $frameParameters->attribute('strut4Ring2HoleId');
         $ringHoleId[0][4] = $frameParameters->attribute('strut5Ring1HoleId');
         $ringHoleId[1][4] = $frameParameters->attribute('strut5Ring2HoleId');
         $ringHoleId[0][5] = $frameParameters->attribute('strut6Ring1HoleId');
         $ringHoleId[1][5] = $frameParameters->attribute('strut6Ring2HoleId');
      }
   }

   # assemble arguments for call of setstrutlengths
   $arguments = '';

   if ($valid) {
      # output ring parameters
      foreach $ringId (@ringId) {
         $file = "../../config/ringparameters/$ringId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $ringParameters = parseXml($xml);
            $arguments .= $ringParameters->attribute('innerRadius') . ' ';
            $arguments .= $ringParameters->attribute('outerRadius') . ' ';
            $arguments .= $ringParameters->attribute('thickness') . ' ';
            $arguments .= $ringParameters->attribute('holeCircleRadius') . ' ';
            $arguments .= $ringParameters->attribute('holeCount') . ' ';
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for ring <i>$ringId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output strut parameters
      $i = 0;
      foreach $strutId (@strutId) {
         $file = "../../config/strutparameters/$strutId.xml";
         if (-f $file) {
            $xml = importFile($file);
            $strutParameters = parseXml($xml);
            $minLength = $strutParameters->attribute('minLength');
            $maxLength = $strutParameters->attribute('maxLength');
            $arguments .= $minLength . ' ';
            $arguments .= $maxLength . ' ';
            $arguments .= $strutParameters->attribute('screwRadius') . ' ';
            $arguments .= $strutParameters->attribute('screwLength') . ' ';
            $arguments .= $strutParameters->attribute('nutRadius') . ' ';
            $arguments .= $strutParameters->attribute('nutLength') . ' ';
            $arguments .= $strutParameters->attribute('tubeRadius') . ' ';
            $arguments .= $strutParameters->attribute('tubeLength') . ' ';
            $arguments .= $strutParameters->attribute('jointRadius') . ' ';
            $arguments .= $strutParameters->attribute('jointOffset') . ' ';

            # check strut lengths against limits
            if ($strutLength[$i] < $minLength) {
               $valid = 0;
               $save = 0;
               $j = $i + 1;

               $message = <<EOT;
Strut $j length ($strutLength[$i]) is less than the minimum length of this
strut ($minLength). Please correct this.
EOT

            } elsif ($strutLength[$i] > $maxLength) {
               $valid = 0;
               $save = 0;
               $j = $i + 1;

               $message = <<EOT;
Strut $j length ($strutLength[$i]) is greater than the maximum length of this
strut ($maxLength). Please correct this.
EOT

            }
            $i++;
         } else {
            $valid = 0;

            $message = <<EOT;
Saved parameters for strut <i>$strutId</i> not found.
EOT

            last;
         }
      }
   }

   if ($valid) {
      # output required frame parameters
      for $i (0..5) {
         $arguments .= $ringHoleId[0][$i] . ' ';
         $arguments .= $ringHoleId[1][$i] . ' ';
      }

      # output strut length parameters
      foreach $strutLength (@strutLength) {
         $arguments .= $strutLength . ' ';
      }
      $arguments .= $ringSpacing . ' ';

      # get temporary file name suffix
      $fileSuffix = mktemp('-XXXXXX');
      $arguments .= $fileSuffix . ' ';

      # generate graphics and transformations files
      $command = "../../bin/setstrutlengths $arguments >> log$fileSuffix";  
      $status = system($command);  
      if ($status == 255) {  # system() returned -1
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: system call failed.
EOT

      } elsif (($status >> 8) == 255) {  # setstrutlengths returned -1
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: The ring position computation is not converging. Are all
parameters correctly defined? Are ring 1 and ring 2 hole numbers approximately
aligned?
EOT

      } elsif (!((-f "graphics/frontview$fileSuffix.png") && 
                 (-f "graphics/sideview$fileSuffix.png") &&
                 (-f "transformations$fileSuffix"))) {
         $valid = 0;
         $save = 0;

         $message = <<EOT;
Software problem: generated files not found.
EOT

      } else {
         # read ring transformations from file
         $file = "transformations$fileSuffix";
         open(FILE, "$file") or die("can't open $file for write");
         for $i (0..1) {
            $line = <FILE>;
            for $j (0..15) {
               $line =~ m/\G([^ ]+)\s+/g;
               $transformations[$i][$j] = $1;
            }
         }

         $message = <<EOT;
Front and side views of the frame with the given strut lengths are shown below.
EOT

      }
   }

   if ($save) {
      if ($frameId ne 'example-frame') {
         # save strut length parameters
         $file = "../../config/strutlengths/$frameId.xml";
         open(FILE, ">$file") or die("can't open $file for write");

         $xml = <<EOT;
<strutLengths
  id="$frameId" strut1Length="$strutLength[0]"
  strut2Length="$strutLength[1]" strut3Length="$strutLength[2]"
  strut4Length="$strutLength[3]" strut5Length="$strutLength[4]"
  strut6Length="$strutLength[5]" ringSpacing="$ringSpacing"
EOT

         for $i (1..2) {
            $xml .= "  ring${i}Transform=\"";
            for $j (0..15) {
               $xml .= "$transformations[$i - 1][$j] ";
            }
            chop($xml);
            $xml .= "\"\n";
         }
         $xml .= "/>\n";

         print(FILE $xml);
         close(FILE) or die("can't open $file for write");
      }
   }

   # output new html page
   print <<EOT;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Strut Lengths</title>
<link rel="stylesheet" type="text/css" href="imports/style.css"/>
</head>
<body>
<div id="page-main">
<div id="page-header">
<div id="header-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Strut Lengths

</div>
<div id="header-nav">
<a href="frameparameters.html">Previous</a><span class="separator"> I </span><a href="xrayparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
<div id="root-prefix">
<a href="../../../../lme/html/index.html">Laboratory for Micro Enterprise</a><span class="separator"> &gt; </span><a href="../../../../lme/html/camlab.html">CAM Lab</a></div>
<div id="root-title">
<a href="index.html">Hexapod Fixator</a></div>
<div id="header-menu">
<a href="ringparameters.html">Ring&nbsp;Parameters</a><span class="separator"> I </span><a href="strutparameters.html">Strut&nbsp;Parameters</a><span class="separator"> I </span><a href="frameparameters.html">Frame&nbsp;Parameters</a><span class="separator"> I </span><a href="strutlengths.html">Strut&nbsp;Lengths</a><span class="separator"> I </span><a href="xrayparameters.html">X-Ray&nbsp;Parameters</a><span class="separator"> I </span><a href="deformityparameters.html">Deformity&nbsp;Parameters</a><span class="separator"> I </span><a href="deformitycorrection.html">Deformity&nbsp;Correction</a><span class="separator"> I </span><a href="software.html">Software</a>
</div>
</div>
<div id="page-contents">
<h1>Strut Lengths</h1>
EOT

   # output message
   if ($valid) {

      print <<EOT;
<div class="boxed">
$message
</div>

EOT

   } else {

      print <<EOT;
<div class="boxed">
<span class="warn">$message</span>
</div>

EOT

   }
                                                     
   print <<EOT;
<div class="section">

<p>When frame parameters have been defined, and the frame has been fitted to
the patient, enter initial strut lengths and approximate ring spacing here. The
software will then compute the initial frame geometry, and generate views of
the frame. Note that this may take some time.</p>

<p>If you enter a frame identifier for which you have previously defined
initial strut lengths and approximate ring spacing, and leave the data fields
empty, then the saved data will be used and can be edited of required. Enter
<i>example-frame</i> for example definitions.</p>

<form action="strutlengths.cgi" method="GET">
<input name="_submitted" type="hidden" value="1">
<input name="_sessionid" type="hidden" value="999">

<fieldset>

<p><input name="frameId" size="30" type="text" value="$frameId">Frame identifier</p>

</fieldset>

<p>The strut lengths required are the precise distances between universal joint
centres. The approximate ring centre-to-centre spacing helps the least-squares
search algorithm to localise the solution to the forward kinematics problem. If
in doubt enter say 70% of the maximum strut length. Dimension units are mm.</p>

<fieldset>

<p><input name="strut1Length" size="6" type="text" value="$strutLength[0]">Strut 1 length</p>

<p><input name="strut2Length" size="6" type="text" value="$strutLength[1]">Strut 2 length</p>

<p><input name="strut3Length" size="6" type="text" value="$strutLength[2]">Strut 3 length</p>

<p><input name="strut4Length" size="6" type="text" value="$strutLength[3]">Strut 4 length</p>

<p><input name="strut5Length" size="6" type="text" value="$strutLength[4]">Strut 5 length</p>

<p><input name="strut6Length" size="6" type="text" value="$strutLength[5]">Strut 6 length</p>

<p><input name="ringSpacing" size="6" type="text" value="$ringSpacing">Approximate ring 
centre-to-centre spacing</p>

</fieldset>

<center><button type="submit">Submit</button></center>

</form>
</div>

EOT

   if ($valid) {

      print <<EOT;
<p><center>
<img src="graphics/frontview$fileSuffix.png"></img>
<img src="graphics/sideview$fileSuffix.png"></img>
</center></p>

EOT

   }

   print <<EOT;
</div>
<div id="page-footer">
<div id="footer-path">
<a href="index.html">Home</a><span class="separator"> &gt; </span>Strut Lengths

</div>
<div id="footer-nav">
<a href="strutlengths.html">Top</a><span class="separator"> I </span><a href="frameparameters.html">Previous</a><span class="separator"> I </span><a href="xrayparameters.html">Next</a><span class="separator"> I </span><a href="sitemap.html">Index</a>
</div>
</div>
</div>
</body>
</html>
EOT

}
