
// Copyright 1996 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#include "primitive.h"
#include "artifact.h"


void 
ArtifactInstance::setTransformation
  (const Vector3 rotation,
   const Vector3 translation)
{
   this->rotation = rotation;
   this->translation = translation;
}


// class Artifact member functions

Artifact::Artifact()
{
   firstArtifact = 0;
   firstPrimitive = 0;
}

Artifact::Artifact
  (const char* artifactName)
{
   firstArtifact = 0;
   firstPrimitive = 0;
   
   int i = 0;
   char c;
   do
      {
	 c = name[i] = artifactName[i];
	 i++;
      }
   while (c != 0);
}

Artifact::~Artifact()
{
   // delete artifact and primitive instances
}

ArtifactInstance*
Artifact::add
  (Artifact* instance)
{
   ArtifactInstance* newArtifact = new ArtifactInstance;
   newArtifact->artifact = instance;
   newArtifact->rotation = Vector3(0, 0, 0);
   newArtifact->translation = Vector3(0, 0, 0);
   newArtifact->next = firstArtifact;
   firstArtifact = newArtifact;
   return newArtifact;
}

ArtifactInstance*
Artifact::add
  (Artifact* instance, 
   const Vector3 translation)
{
   ArtifactInstance* newArtifact = new ArtifactInstance;
   newArtifact->artifact = instance;
   newArtifact->rotation = Vector3(0, 0, 0);
   newArtifact->translation = translation;
   newArtifact->next = firstArtifact;
   firstArtifact = newArtifact;
   return newArtifact;
}

ArtifactInstance*
Artifact::add
  (Artifact* instance, 
   const Vector3 rotation,
   const Vector3 translation)
{
   ArtifactInstance* newArtifact = new ArtifactInstance;
   newArtifact->artifact = instance;
   newArtifact->rotation = rotation;
   newArtifact->translation = translation;
   newArtifact->next = firstArtifact;
   firstArtifact = newArtifact;
   return newArtifact;
}

ArtifactInstance*
Artifact::add  
  (Artifact* instance,  
   const Vector3 local1,  // local maps to global
   const Vector3 global1,
   const Vector3 local2,
   const Vector3 global2)

// Rotation about x axis - only y and z coordinates are affected. View axes 
// with positive y to the right, positive z up. Then positive x is away from 
// viewer, and positive rotations are anticlockwise.
//
// Rotation about y axis - only x and z coordinates are affected. View axes 
// with positive z to the right, positive x up. Then positive y is away from 
// viewer, and positive rotations are anticlockwise.
//
// Rotation about z axis - only x and y coordinates are affected. View axes 
// with positive x to the right, positive y up. Then positive z is away from 
// viewer, and positive rotations are anticlockwise.

{
   ArtifactInstance* newArtifact = new ArtifactInstance;
   newArtifact->artifact = instance;
   newArtifact->next = firstArtifact;
   firstArtifact = newArtifact;

   Vector3 translation (global1 - local1);
   Vector3 local (local2 - local1);
   Vector3 global (global2 - global1);
   Vector3 rotation (local.rotationsToXXX(global));
   newArtifact->rotation = rotation;
   newArtifact->translation = translation;

   return newArtifact;
}
	    
PrimitiveInstance*
Artifact::add
  (Primitive* instance)
{
   PrimitiveInstance* newPrimitive = new PrimitiveInstance;
   newPrimitive->primitive = instance;
   newPrimitive->next = firstPrimitive;
   firstPrimitive = newPrimitive;
   return newPrimitive;
}

void
Artifact::display
  (int /*indent = 0*/)
{
   /* must be changed - pointer linked list
   for (int j = 0; j < indent; j++)
      cout << ' ';
   cout << name << '\n';

   for (int i = 1; i <= artifactCount; i++)
      artifactArray[i]->display(indent + 3);
      */
}

void
Artifact::emitPovraySource
  (ofstream &povraySource, Matrix4 transformation)
{
   povraySource << "// artifact " << name << '\n';

   // first descend the composition tree
   ArtifactInstance* artifactInstance = firstArtifact;
   while (artifactInstance)
      {
	 Matrix4 newTransformation 
	   (transformation, 
	    artifactInstance->rotation,
	    artifactInstance->translation); 
         artifactInstance->artifact->emitPovraySource
                                        (povraySource, newTransformation);
	 artifactInstance = artifactInstance->next;
         povraySource << "\n";
      }

   // now the local primitives
   PrimitiveInstance* primitiveInstance = firstPrimitive;
   while (primitiveInstance)
      {
         primitiveInstance->primitive->emitPovraySource
                                          (povraySource, transformation);
	 primitiveInstance = primitiveInstance->next;
      }
}
