
// Copyright 1996 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#ifndef ArtifactClasses
#define ArtifactClasses

using namespace std;

#include <fstream>

#include "Vector3.H"
#include "Matrix4.H"

using namespace std;

class Artifact;

class ArtifactInstance
{
   public:
      void setTransformation
        (const Vector3 rotation,
         const Vector3 translation);
   private:
      Artifact* artifact;
      Vector3 rotation;
      Vector3 translation;
      ArtifactInstance* next;
      // must be a doubly linked list so we can 
      // easily remove an instance given a pointer to it.
      // if we also have a head and tail pointer, can
      // scan the list in either direction, and emit
      // instances is the order of adding.
				    
   friend class Artifact;
};

class PrimitiveInstance
{
   private:
      Primitive* primitive;
      PrimitiveInstance* next;

   friend class Artifact;
};

class Artifact {
public:
   Artifact();
   Artifact
     (const char* name);
   ~Artifact();
   ArtifactInstance* add
     (Artifact* instance);
   ArtifactInstance* add
     (Artifact* instance, 
      const Vector3 translation); 
   ArtifactInstance* add
     (Artifact* instance, 
      const Vector3 rotation,
      const Vector3 translation); 
   ArtifactInstance* add
     (Artifact* instance,
      const Vector3 local1,  // local maps to global
      const Vector3 global1,
      const Vector3 local2,
      const Vector3 global2);
   PrimitiveInstance* add
     (Primitive* instance);

   void display
     (int /*indent = 0*/);
   void emitPovraySource
     (ofstream &povraySource, Matrix4 transformation); // const dropped

protected:
   char name[20];
   ArtifactInstance* firstArtifact;
   PrimitiveInstance* firstPrimitive;
};

#endif
