
// Copyright 1996 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#include "primitive.h"


// class Primitive member functions

void 
Primitive::emitPovraySource
  (ofstream &povraySource, const Matrix4)
{
   povraySource 
      << "emitPovraySource - virtual function not defined for primitive\n";
}


// class Cylinder member functions

Cylinder::Cylinder
  (const Vector3 end1, const Vector3 end2, const double radius,
   const char* texture)
{
   this->end1 = end1;
   this->end2 = end2;
   this->radius = radius;
   this->texture = texture;
}

void 
Cylinder::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "cylinder {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius;
   povraySource << "\n";
   povraySource << "   texture { " << texture << " }\n";
   povraySource << "}\n";
}


// class Sphere member functions

Sphere::Sphere
  (const Vector3 centre, const double radius,
   const char* texture )
{
   this->centre = centre;
   this->radius = radius;
   this->texture = texture;
}

void 
Sphere::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "sphere {\n";
   povraySource << "   ";
   centre.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius;
   povraySource << "\n";
   povraySource << "   texture { " << texture << " }\n";
   povraySource << "}\n";
}


// class Box member functions

Box::Box
  (const Vector3 corner1, const Vector3 corner2, const char* texture)
{
   this->corner1 = corner1;
   this->corner2 = corner2;
   this->texture = texture;
}

void 
Box::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "box {\n";
   povraySource << "   ";
   corner1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   corner2.emitPovraySource(povraySource, transformation);
   povraySource << "\n";
   povraySource << "   texture { " << texture << " }\n";
   povraySource << "}\n";
}


// class Cone member functions

Cone::Cone
  (const Vector3 end1, const double radius1,
   const Vector3 end2, const double radius2,
   const char* texture)
{
   this->end1 = end1;
   this->end2 = end2;
   this->radius1 = radius1;
   this->radius2 = radius2;
   this->texture = texture;
}

void 
Cone::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "cone {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius1;
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << radius2;
   povraySource << "\n";
   povraySource << "   texture { " << texture << " }\n";
   povraySource << "}\n";
}


// class OpenCylinder member functions

OpenCylinder::OpenCylinder
  (const Vector3 end1, const Vector3 end2, 
   const double outerRadius, const double innerRadius, const char* texture)
{
   this->end1 = end1;
   this->end2 = end2;
   this->outerRadius = outerRadius;
   this->innerRadius = innerRadius;
   this->texture = texture;
}

void 
OpenCylinder::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "difference {\n";

   povraySource << "cylinder {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << outerRadius;
   povraySource << "\n";
   povraySource << "   texture { " << texture << " }\n";
   povraySource << "}\n";

   povraySource << "cylinder {\n";
   povraySource << "   ";
   end1.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   end2.emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   povraySource << innerRadius;
   povraySource << "\n";
   povraySource << "open\n";
   povraySource << "   texture { " << texture << " }\n";
   povraySource << "}\n";

   povraySource << "}\n";
}


// experimental

// class Union member functions

Union::Union
  (Primitive& prim1, Primitive& prim2)
{
   primitive1 = &prim1;
   primitive2 = &prim2;
}

void 
Union::emitPovraySource
  (ofstream &povraySource, const Matrix4 transformation)
{
   povraySource << "union {\n";
   povraySource << "   ";
   primitive1->emitPovraySource(povraySource, transformation);
   povraySource << ", ";
   primitive2->emitPovraySource(povraySource, transformation);
   povraySource << "\n";
   povraySource << "   texture { MyTexture1 }\n";
   povraySource << "}\n";
}
