
// Copyright 1996 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#ifndef PrimitiveClasses
#define PrimitiveClasses

using namespace std;

//#include <math.h>
#include <fstream>

#include "Vector2.H"
#include "Vector3.H"
#include "Matrix4.H"

using namespace std;

class Primitive
{
   public:
      Primitive() {};
      virtual ~Primitive() {};
      virtual void emitPovraySource
        (ofstream &povraySource, const Matrix4);
};


// Cylinder primitive

class Cylinder: public Primitive
{
   public:
      Cylinder() {};
      Cylinder
        (const Vector3 p1, const Vector3 p2, const double rad,
         const char* texture);
      void emitPovraySource
        (ofstream &povraySource, const Matrix4 transformation);
   private:
      Vector3 end1, end2;
      double radius;
      const char* texture;
};


// Sphere primitive

class Sphere: public Primitive
{
   public:
      Sphere() {};
      Sphere
        (const Vector3 p, const double rad,
         const char* texture);
      void emitPovraySource
        (ofstream &povraySource, const Matrix4 transformation);
   private:
      Vector3 centre;
      double radius;
      const char* texture;
};


// Box primitive

class Box: public Primitive
{
   public:
      Box() {};
      Box
        (const Vector3 p1, const Vector3 p2,
         const char* texture);
      void emitPovraySource
        (ofstream &povraySource, const Matrix4 transformation);
   private:
      Vector3 corner1, corner2;
      const char* texture;
};


// Cone primitive

class Cone: public Primitive
{
   public:
      Cone() {};
      Cone
        (const Vector3 p1, const double rad1,
         const Vector3 p2, const double rad2,
         const char* texture);
      void emitPovraySource
        (ofstream &povraySource, const Matrix4 transformation);
   private:
      Vector3 end1, end2;
      double radius1, radius2;
      const char* texture;
};


// OpenCylinder primitive

class OpenCylinder: public Primitive
{
   public:
      OpenCylinder() {};
      OpenCylinder
        (const Vector3 p1, const Vector3 p2, 
         const double outerRadius, const double innerRadius,
         const char* texture);
      void emitPovraySource
        (ofstream &povraySource, const Matrix4 transformation);
   private:
      Vector3 end1, end2;
      double outerRadius, innerRadius;
      const char* texture;
};


// experimental

class Union: public Primitive
{
   public:
      Union() {};
      Union
        (Primitive& prim1, Primitive& prim2);
      void emitPovraySource
        (ofstream &povraySource, const Matrix4 transformation);
   private:
      Primitive *primitive1, *primitive2;
};

#endif
