// Copyright 2007 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#ifndef FixatorFrameClass
#define FixatorFrameClass

#include "Vector3.H"
#include "fixatorring.h"
#include "fixatorstrut.h"


struct XrayParameters
{
   double xrayDistanceFrontal;
   double ring0NearestFrontal;
   double ring1NearestFrontal;
   double xrayDistanceLateral;
   double ring0NearestLateral;
   double ring1NearestLateral;
};

struct DeformityParameters
{
   double x1Ring0Frontal;
   double z1Ring0Frontal;
   double x2Ring0Frontal;
   double z2Ring0Frontal;
   double x1Ring1Frontal;
   double z1Ring1Frontal;
   double x2Ring1Frontal;
   double z2Ring1Frontal;
   double x1Ring0Lateral;
   double z1Ring0Lateral;
   double x2Ring0Lateral;
   double z2Ring0Lateral;
   double x1Ring1Lateral;
   double z1Ring1Lateral;
   double x2Ring1Lateral;
   double z2Ring1Lateral;
   double axialExtension;
   double axialRotation;
};

struct GraphicsParameters
{
   int imageWidth;
   int imageHeight;
   int pixelsPerInch;
   int gridPitch;
   bool showOrigin;
   bool showTarget;
   bool showNearest;
   bool showGrid;
   bool showBones;
};

class FixatorFrame
{
   public:
      FixatorFrame();
      void setXrayParameters(const XrayParameters &parameters);
      void setDeformityParameters(const DeformityParameters &parameters);
      void setGraphicsParameters(const GraphicsParameters &parameters);
      void transformRingTo(const int ringId, const Vector3 &rotation,
                           const Vector3 &translation);
      void transformRingBy(const int ringId, const Vector3 &rotation, 
                           const Vector3 &translation);
      void transformBy(const Vector3 &rotation, const Vector3 &translation);
      void transformBy(const Matrix4 &transformation);
      int setStrutLengths(const double strutLength[6], const double approxZ);
      void transformView(const double ringNearest[2]);
      void computeBoneCoords();
      double correctDeformity(const int totalSteps, const int stepNumber);
      void emitPovraySource(ofstream &povraySource, const double xrayDistance);

      XrayParameters xrayParameters;
      DeformityParameters deformityParameters;
      GraphicsParameters graphicsParameters;
      FixatorRing ring[2];
      FixatorStrut strut[6];
      Vector3 xrayTargetCoords;
};

#endif
