// Copyright 2007 John Storrs. All rights reserved. This work may be copied, 
// modified and distributed under the terms of the GNU General Public License.

#ifndef FixatorRingClass
#define FixatorRingClass

#include <fstream>
#include "Vector3.H"
#include "Matrix4.H"

// forward declaration
class FixatorStrut;

const int maxHoles = 200;

struct RingParameters {
   double innerRadius;
   double outerRadius; 
   double thickness;
   double jointOffset[6];
   double jointRadius[6];
   double boneRadius;
   double holeCircleRadius;
   int holeCount;
};

struct Coords {
   Vector3 centreCoords;   // ring centre
   Vector3 ringCoords[6];  // base of joint support
   Vector3 jointCoords[6]; // joint centre
   Vector3 nearestCoords;  // x-ray parameter
   Vector3 boneCoords[3];  // deformity parameters
};

class FixatorRing
{
   public:
      FixatorRing();
      void setParameters(const int id, const RingParameters &parameters);
      void connectStrut(FixatorStrut *strut, const int Holeid);
      void setLocalBoneCoords(const Vector3 &point0, const Vector3 &point1, 
                              const Vector3 &point2);
      void transformTo(const Vector3 &rotation, const Vector3 &translation);
      void transformBy(const Vector3 &rotation, const Vector3 &translation);
      void transformTo(const Matrix4 &transformation);
      void transformBy(const Matrix4 &transformation);
      Vector3 yMaxCoords();
      void emitPovraySource(ofstream &povraySource);

      int id;  // 0..1
      int showNearest;
      int showBone;
      RingParameters parameters;
      Vector3 holeCoords[maxHoles];
      Matrix4 transformation;
      Coords local, global;
      FixatorStrut *strut[6];
};

#endif
